#!/bin/bash

###################################################
# 功能: 记录日志
#
# 接口:
#   函数: error <log>
#   函数: warn <log>
#   函数: info <log>
#   函数: debug <log>
#
# 输入: 日志信息
#
# 示例:
#   error "run failed"
###################################################

g_local_ip="$(echo ${SSH_CONNECTION} | awk '{print $3}')"

####################################################################
#  FUNCTION     : mv_log_file
#  DESCRIPTION  : mv log file, if file count bigger than 10, 
#                  then delete the oldest file
#  INPUT        : 
#  OUTPUT       : 
#  RETURN       : 
####################################################################
function mv_log_file()
{
    for file in $(ls -vr $g_log_file.[[:digit:]]* 2> /dev/null)
    do
        local idx=${file#${g_log_file}.}
        if [ $idx -gt 20 ];then
            rm -f $file
            continue
        fi
        
        idx=$((idx+1))
        mv "$file" "${g_log_file}.$idx"
    done
    
    mv "${g_log_file}" "${g_log_file}.1"
}

####################################################################
#  FUNCTION     : rotate_log_file
#  DESCRIPTION  : rotate log file, if file size bigger than 20MB
#  INPUT        : 
#  OUTPUT       : 
#  RETURN       : 
####################################################################
function rotate_log_file()
{
    local logSize=`du -ms $g_log_file |cut -f1`
    if [ $logSize -gt 60 ]; then
        #如果日志文件大于60M，则备份日志文件
        mv_log_file
    fi
}

###################################################
# 把内容写入日志文件
###################################################
function _write_logfile()
{
    local local_ip=""
    
    # g_log_file 为空,则默认设置为 ${HOME}/fi-preinstall.log
    if [ -z "$g_log_file" ]; then
        g_log_file="${HOME}/fi-preinstall.log"
    fi

    #如果日志文件已经产生,可以直接写入(绝大部分情况符合此场景)
    if [ -f "${g_log_file}" ]; then
        echo "[$(date)][${g_local_ip}] $*" >> "${g_log_file}"
        rotate_log_file
        return 0
    fi

    #日志文件不存在,日志文件所在的目录已经存在,可以直接写入
    if [ -d "$(dirname ${g_log_file})" ]; then
        echo "[$(date)][${g_local_ip}] $*" >> "${g_log_file}"
        return 0
    fi

    #日志文件所在的目录不存在
    mkdir -p "$(dirname ${g_log_file})" > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        #目录创建成功,可以写入日志
        echo "[$(date)][${g_local_ip}] $*" >> "${g_log_file}"
        return 0
    fi

    return 1
}

###################################################
# 检查日志文件的权限
###################################################
function check_logfile()
{
    # g_log_file 为空,则默认设置为 ${HOME}/fi-preinstall.log
    if [ -z "$g_log_file" ]; then
        g_log_file="${HOME}/fi-preinstall.log"
    fi
    
    if [ ! -e "${g_log_file}" ]; then
        return 0
    fi

    #如果日志文件已经产生,可以直接写入(绝大部分情况符合此场景)
    if [ -f "${g_log_file}" -a -r "${g_log_file}" -a -w "${g_log_file}" ]; then
        return 0
    fi

    #日志文件存在且没有可写权限，返回失败
    echo -e "\033[31mError: ${g_log_file}: Permission denied, Please check your permission or clean ${HOME}/fi-preinstall.log and /tmp/diskmgt/\033[0m"
    return 1
}

###################################################
# 记录错误日志
###################################################
function error()
{
    # error日志 同时在界面上输出(红色)
    echo -e "\033[31mERROR: $*\033[0m"

    #输出error日志
    _write_logfile "[ERROR]" "$@"
}

###################################################
# 记录警告日志
###################################################
function warn()
{
    # error日志 同时在界面上输出(黄色)
    echo -e "\033[33mWARN: $*\033[0m"

    #输出error日志
    _write_logfile "[WARN]" "$@"
}

###################################################
# 记录一般日志(不往界面输出)
###################################################
function info()
{
    #输出info日志
    _write_logfile "[INFO]" "$@"
}

###################################################
# 记录debug(不往界面输出)
###################################################
function debug()
{
    #如果 g_debug 没有定义,则直接返回
    if [ -z "${g_debug}" ]; then
        return
    fi

    #输出debug日志
    if [ "${g_debug}" -eq 1 ]; then
        _write_logfile "[DEBUG]" "$@"
    fi
}

