#!/bin/bash

###################################################
# 功能: 解析主机名
#
# 接口: (函数) parse_host <a list of hostname>
#
# 输入: 主机名(或IP地址)列表字符串
# 输出: 以逗号为分割符的主机名(或IP地址)列表
#
# 示例:
#   parse_host "192.168.23.[11,14-15]"
###################################################

###################################################
#  主机名格式化说明：
#  1:不同的主机名或IP地址以逗号(英文逗号)为分割符
#   例如: 192.168.23.11,host11
#  2:使用方括号扩展IP地址或主机名
#   例如: 192.168.23.[11-13] 表示 192.168.23.11,192.168.23.12,192.168.23.13
#         host[1-4] 表示 host1,host2,host3,host4
#         192.168.23.[11,14-15] 表示 192.168.23.11,192.168.23.14,192.168.23.15
#         host[a,b,c] 表示 hosta,hostb,hostc
#  3:一个IP地址或主机名表达式可以包含多个方括号
#   例如: 10.[85-86].[10-15,80].[12,50-54]
#  4:扩展后的主机名以逗号为分割符
###################################################

###################################################
#
# 检查host字符串
# 1:判断HOST是否为空字符串
# 2:判断是否有其他非法字符
# 3:判断方括号是否成对出现
# 4:判断是否包含空格,重复的逗号和中横线,空白方括号
#
###################################################
function _check_host()
{
    local host="$1"    #主机名或IP地址

    debug "[${host}] Enter [${FUNCNAME}:${LINENO}]"

    #判断HOST是否为空字符串
    if [ -z "$host" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Host is Null"
        return 1
    fi

    #判断是否有其他非法字符
    if [ -n "$(echo "$host" | tr -d [:alnum:][:alpha:][].,-:)" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Host($host) contains forbidden character"
        return 1
    fi

    #判断方括号是否成对出现
    if [ -n "$(echo "$host" | tr -cd [] | sed 's/\[\]//g')" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Host($host) contains non-matching brackets"
        return 1
    fi

    #判断是否包含空格,重复的逗号和中横线,空白方括号
    if [ -n "$(echo "$host" | grep -E ',,|--|\[\]|[[:space:]]')" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Host($host) contains invalid combination character"
        return 1
    fi

    debug "[${host}] Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 扩展横线(-)为逗号(,) 例如:把 [1,3-5] 转换 [1,3,4,5]
#
###################################################
function _extend_horizontal()
{
    local host="$1"    #主机名或IP地址
    local result="$1"  #转换后的主机名或IP地址
    local b_num=0      #方括号(brackets)个数
    local i=0          #循环变量
    local j=0          #循环变量
    local start=0      #中横线左侧的数字
    local end=0        #中横线右侧的数字
    local tmp=""       #扩展过程的中间变量

    debug "[${host}] Enter [${FUNCNAME}:${LINENO}]"

    if [ -z "$host" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Host is Null"
        return 1
    fi

    b_num=$(echo "$host" | tr -cd [ | wc -c)
    if [ $b_num -eq 0 ]; then
        #没有方括号 不需要扩展
        echo "$host"
        return 0
    fi

    for((i=2;i<2+$b_num;i++))
    do
        while read line
        do
            start=$(echo "$line" | awk -F'-' '{print $1}')
            end=$(echo "$line" | awk -F'-' '{print $2}')
            if [ -n "$(echo "${start}" | tr -d [:digit:])" ] || \
               [ -n "$(echo "${end}" | tr -d [:digit:])" ] || [ $start -ge $end ]; then
                error "[${FUNCNAME}:${LINENO}]" "horizontal format invalid[$line]"
                return 1
            fi
            tmp="$start"
            for ((j=$start + 1; j<=$end; j++))
            do
                tmp="${tmp},$j"
            done
            result=$(echo "$result" | sed 's/\['$line'/\['$tmp'/g' | sed 's/,'$line'/,'$tmp'/g')
        done < <(echo "$host" | awk -F'[' '{print $'$i'}' | awk -F']' '{print $1}' \
                              | tr ',' '\n' | awk '{if ($1~/-/) print $1}')
    done

    debug "[${result}] Leave [${FUNCNAME}:${LINENO}]"

    echo "$result"
    return 0
}

###################################################
#
# 扩展方括号,即消除host中的方括号
#
###################################################
function _extend_brackets()
{
    local host="$1"               #主机名或IP地址
    local result=""               #转换后的主机名
    local b_num=0                 #方括号(brackets)个数
    local all_num=0               #扩展后总的主机名个数
    local element=""              #保存数组元素
    local e_num=""                #数组元素中的子项个数
    local e_index=0               #数组元素中的子项的索引值
    local e_value=""              #数组元素中的子项的内容
    local h_tmp=""                #主机名临时变量
    local i=0                     #循环变量
    local j=0                     #循环变量
    local k=0                     #循环变量
    declare -a name_arr=()        #方括号内的名字数组
    declare -a num_arr=()         #方括号内的名字的个数

    debug "[${host}] Enter [${FUNCNAME}:${LINENO}]"

    #入参检查
    if [ -z "$host" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Host is Null"
        return 1
    fi

    #获取方括号的个数
    b_num=$(echo "$host" | tr -cd [ | wc -c)
    if [ $b_num -eq 0 ]; then
        #没有方括号不需要扩展
        echo "$host"
        return 0
    fi

    #解析方括号的内容
    all_num=1
    for ((i=2;i<2+$b_num;i++))
    do
        element=$(echo "$host" | awk -F'[' '{print $'$i'}' | awk -F']' '{print $1}')
        name_arr=(${name_arr[@]} $element)
        e_num=$(($(echo "$element" | tr -cd ',' | wc -c) + 1))
        num_arr=(${num_arr[@]} $e_num)
        all_num=$(($all_num * $e_num))
    done

    #扩展方括号的内容
    for ((i=0;i<$all_num;i++))
    do
        k=1
        h_tmp="$host"
        for ((j=0;j<${#name_arr[@]};j++))
        do
            e_index=$((($i % ($k * ${num_arr[$j]})) / $k + 1))
            e_value=$(echo "${name_arr[$j]}" | awk -F',' '{print $'$e_index'}')
            k=$(($k * ${num_arr[$j]}))
            h_tmp=$(echo "$h_tmp" | sed 's/\['${name_arr[$j]}'\]/'$e_value'/')
        done
        result="${result},${h_tmp}"
    done

    #判断目标主机名是否为空
    if [ -z "$result" ]; then
        error "[${FUNCNAME}:${LINENO}]" "result is Null"
        return 1
    fi

    #删除字符串最前面的逗号
    result="$(echo "$result" | sed "s/^,//g")"

    debug "[${result}] Leave [${FUNCNAME}:${LINENO}]"

    echo "$result"
    return 0

}

###################################################
#
# 解析HOST: 共分为6个步骤
#  1:检查host字符串
#  2:如果没有方括号 不需要扩展 直接返回
#  3:把中括号([])以外的逗号(,)转换为分号(;)
#  4:扩展中横线(-)为逗号(,) 例如:把 [1,3-5] 转换 [1,3,4,5]
#  5:以分号为分隔符,对每个host单独处理
#  6:扩展队列中的每个元素的中括号([])
#  7:生成一个以逗号为分割符的字符串
#  8:去除重复的host
#
###################################################
function parse_host()
{
    local host="$1"     #主机名或IP地址
    local result=""     #解析后的主机名
    local h_tmp=""      #保存临时主机名

    debug "[${host}] Enter [${FUNCNAME}:${LINENO}]"

    #1:检查host字符串
    _check_host "$host"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "_check_host failed"
        return 1
    fi

    #2:如果没有方括号 不需要扩展
    if [ -z "$(echo "$host" | grep '\[')" ]; then
        host="$(echo "$host" | tr ',' '\n' | sort | uniq | tr '\n' ',' | sed "s/^,//g" | sed "s/,$//g")"
        echo $host
        return 0
    fi

    #3:把中括号([])以外的逗号(,)转换为分号(;)
    host=$(echo "$host" | awk 'BEGIN{FS=""} {j=1;
        for(i=1;i<=NF;i++){
            if ($i=="[")j=0;
            if ($i=="]")j=1;
            if (($i==",")&&(j == 1))printf ";";
            else printf $i;
        }}')

    #4:扩展中横线(-)为逗号(,) 例如:把 [1,3-5] 转换 [1,3,4,5]
    host=$(_extend_horizontal "$host")
    if [ $? -ne 0 ]; then
        echo "$host"
        error "[${FUNCNAME}:${LINENO}]" "_extend_horizontal failed"
        return 1
    fi

    #5:以分号为分隔符,对每个host单独处理
    while read line
    do
        if [ -z "$line" ]; then
            continue
        fi
        if [ -z "$(echo "$line" | grep '\[.*\]')" ]; then
            result="${result},${line}"
            continue
        fi

        #6:扩展队列中的每个元素的中括号([])
        h_tmp=$(_extend_brackets "$line")
        if [ $? -ne 0 ] || [ -z "$h_tmp" ]; then
            echo "$h_tmp"
            error "[${FUNCNAME}:${LINENO}]" "_extend_brackets(${host}:${line}) failed"
            return 1
        fi
        #7:生成一个以逗号为分割符的字符串
        result="${result},${h_tmp}"
    done < <(echo "$host" | tr ';' '\n' | awk '{print $1}')

    #8:去除重复的host
    host="$(echo "$result" | tr ',' '\n' | sort | uniq | tr '\n' ',' | sed "s/^,//g" | sed "s/,$//g")"
    debug "[${host}] Leave [${FUNCNAME}:${LINENO}]"

    echo "$host"

    return 0
}

