#!/bin/bash

###################################################
# 功能: 获取OS的名称
#
# 接口:
#   函数: get_osname
#
#   备注: _get_osname 为内部接口
###################################################

###################################################
#
# 获取OS的名称
#
# cat /etc/centos-release 
# CentOS release 6.2 (Final)
#
# cat /etc/oracle-release 
# Oracle Linux Server release 6.2
#
# cat /etc/redhat-release 
# Red Hat Enterprise Linux Server release 6.4 (Santiago)
#
# cat /etc/SuSE-release 
# SUSE Linux Enterprise Server 11 (x86_64)
# VERSION = 11
# PATCHLEVEL = 1
#
###################################################
function _get_osname()
{
    local cent_release="/etc/centos-release"
    local ole_release="/etc/oracle-release"
    local rhel_release="/etc/redhat-release"
    local sles_release="/etc/SuSE-release"
    local sles15_release="/etc/os-release"
    local euler_release="/etc/euleros-release"
    local neokylin_relase="/etc/neokylin-release"
    local kylin_productinfo="/etc/.productinfo"
    local bclinux_release="/etc/bclinux-release"
    local openeuler_release="/etc/openEuler-release"
    local kylin_relase="/etc/kylin-release"
    local linxos_release="/etc/linxos-release"
    local name=""

    debug "Enter [${FUNCNAME}:${LINENO}]"
    
    # 北京凝思OS x86，OS命令规则和arm不同，需要适配。凝思6.0.99 arm通过os-release文件通用规则进行解析。
    if [ -f "${linxos_release}" ]; then
        name=$(awk '{print $1"-"$2}' ${linxos_release})
        echo "$name"
        return 0
    fi
        
    #获取Euler的版本, Euleros中存在redhat-release和CentOS-release，最先检查    
    if [ -f "${euler_release}" ]; then
        if grep -q "release" "${euler_release}"; then
            name="$(cat $euler_release | sed 's/x86_64//g' | awk -F " |SP|)" '{print "euleros-"$3"sp"$5}')"
        else
            name="euleros-$(cat "${euler_release}" | awk '{print $2}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
        fi
        echo "$name"
        return 0
    fi
    
    if [ -f "${neokylin_relase}" ];then
        if grep -q "release" "${neokylin_relase}"; then
            name="$(cat "${neokylin_relase}" | awk '{print $6}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
            if [ -z "${name}" ];then
                name="$(cat "${neokylin_relase}" | awk '{print $4}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
            fi
            name="neokylin-$(echo ${name/update/'.'})"
        fi
        echo "$name"
        return 0
    fi
    #获取KylinSecOS的版本
    if [ -f "${kylin_relase}" ] && [ "KylinSec" = "$(cat "${kylin_relase}" |awk -F " " '{print $1}')" ]; then
        name="kylinSecOS-$(grep "^VERSION_ID=" "${sles15_release}" | awk -F "\"" '{print $2}')"
        echo "$name"
        return 0
    fi

    if [ -f "${bclinux_release}" ];then
        if grep -q "Euler" "${bclinux_release}";then
            name="bclinux-$(cat ${bclinux_release} | awk '{print $7}' | awk -F. '{print $1"."$2}')"
        else
            name="bclinux-$(cat "${bclinux_release}" | awk '{print $5}' | awk -F. '{print $1"."$2}')"
        fi
        echo "$name"
        return 0
    fi
    
    #备注: CentOS中也包含 /etc/redhat-release 文件
    #必须在redhat之前检查
    if [ -f "${cent_release}" ]; then
        if grep -q "Linux" "${cent_release}"; then
            name="centos-$(cat "${cent_release}" | awk '{print $4}' | awk -F. '{print $1"."$2}')"
        else
            name="centos-$(cat "${cent_release}" | awk '{print $3}')"
        fi
        echo "$name"
        return 0
    fi

    #备注: oracle linux中也包含 /etc/redhat-release 文件
    #必须在redhat之前检查
    if [ -f "${ole_release}" ]; then
        name="oracle-$(cat "${ole_release}" | awk '{print $5}')"
        echo "$name"
        return 0
    fi

    #获取RedHat的版本
    if [ -f "${rhel_release}" ]; then
        name="redhat-$(cat "${rhel_release}" | awk '{print $7}')"
        echo "$name"
        return 0
    fi

    #获取SuSE的版本
    if [ -f "${sles_release}" ]; then
        name="suse-$(cat "${sles_release}" | grep "^VERSION" | awk '{print $3}')"
        name="${name}.$(cat "${sles_release}" | grep "^PATCHLEVEL" | awk '{print $3}')"
        echo "$name"
        return 0
    fi

    #获取SuSE15的版本
    if [ -f "${sles15_release}" ]; then
        if [ "suse" = "$(cat "${sles15_release}" | grep "^ID_LIKE=" | awk -F "\"" '{print $2}')" ]; then
            name="suse-$(cat "${sles15_release}" | grep "^VERSION_ID=" | awk -F "\"" '{print $2}')"
            echo "$name"
            return 0
        fi
        idLike=$(grep "^ID_LIKE=" "${sles15_release}" | awk -F "\"" '{print $2}')
        ID=$(grep "^ID=" "${sles15_release}" | awk -F '=' '{print $2}' | tr -d '"')
        VERSION_ID=$(grep "^VERSION_ID=" "${sles15_release}" | awk -F "\"" '{print $2}')
        if [ "${ID}" = "kylin" ] && [ "${idLike}" = "fedora" ]; then
            name="$ID-${VERSION_ID}-fedora"
            echo "$name"
            return 0
        fi
        if [ -f "${kylin_productinfo}" ] && [ "${ID}" = "kylin" ]; then
            local MINOR_VERSION=$(grep "${VERSION_ID}" "${kylin_productinfo}" | awk -F "(" '{print $2}' | awk -F ")" '{print $1}')
            name="$ID-${VERSION_ID}-${MINOR_VERSION}"
            echo "$name"
            return 0
        fi
        if [ "${ID}" = "uos" ]; then
            name="$ID-$(cat /etc/os-version | grep MinorVersion | awk -F "=" '{print $2}')"
            echo "$name"
            return 0
        fi
        #获取openEuler-SPx的版本
        if [ -f "${openeuler_release}" ] && [ "${ID}" = "openEuler" ]; then
           local MINOR_VERSION=$(cat "${openeuler_release}" | awk -F "-" '{print $2}' | awk -F ")" '{print $1}')
           #openEuler-20.03-SP1保持之前的规则提取版本号
           local MINOR_VERSION_ID="${VERSION_ID}-${MINOR_VERSION}"
           if [ -n "${MINOR_VERSION}" -a "${MINOR_VERSION_ID}" != "20.03-SP1" ] ; then
               name="$ID-${VERSION_ID}-${MINOR_VERSION}"
               echo "$name"
               return 0
           fi
        fi

        name="$ID-$VERSION_ID"
        echo "$name"
        return 0
    fi	

    debug "Leave [${FUNCNAME}:${LINENO}]"
    return 1
}

###################################################
#
# 功能:  获取远端OS版本
#
# cat /etc/centos-release 
# CentOS release 6.2 (Final)
#
# cat /etc/oracle-release 
# Oracle Linux Server release 6.2
#
# cat /etc/redhat-release 
# Red Hat Enterprise Linux Server release 6.4 (Santiago)
#
# cat /etc/SuSE-release 
# SUSE Linux Enterprise Server 11 (x86_64)
# VERSION = 11
# PATCHLEVEL = 1
#
###################################################
function get_osname_r()
{
    local ipaddr="$1"   #主机名或ip地址
    local cent_release="/etc/centos-release"
    local ole_release="/etc/oracle-release"
    local rhel_release="/etc/redhat-release"
    local sles_release="/etc/SuSE-release"
    local sles15_release="/etc/os-release"
    local euler_release="/etc/euleros-release"
    local name=""
    local version=""

    debug "[${ipaddr}]Enter [${FUNCNAME}:${LINENO}]"

    if [ -z "$ipaddr" ]; then
        error "[${FUNCNAME}:${LINENO}]" "ipaddr is null"
        return 1
    fi
        
    #检查是否为 Euler
    exec_cmd_r "$ipaddr" "120" "ls ${euler_release}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        release_v=$(exec_cmd_r "$ipaddr" "120" "cat ${euler_release}" "2" | grep -i "^EulerOS")
        if echo "$release_v" | grep -q "release"; then
            name="$(echo \"$release_v\" | sed 's/x86_64//g' | awk -F " |SP|)" '{print "euleros-"$3"sp"$5}')"
        else
            name="euleros-$(echo \"$release_v\" | awk '{print $2}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
        fi
        echo $name
        return 0
    fi

    #检查是否为 CentOS
    exec_cmd_r "$ipaddr" "120" "ls ${cent_release}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        version=$(exec_cmd_r "$ipaddr" "120" "cat ${cent_release}" "2")
        if echo "${version}" | grep -q "Linux"; then
            name="centos-$(echo "${version}" | awk '{print $4}' | awk -F. '{print $1"."$2}')"
        else
            name="centos-$(echo "${version}" | awk '{print $3}')"
        fi
        echo $name
        return 0
    fi
    
    #检查是否为 Oracle
    exec_cmd_r "$ipaddr" "120" "ls ${ole_release}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        version=$(exec_cmd_r "$ipaddr" "120" "cat ${ole_release}" "2" | grep "^Oracle Linux" | awk '{print $5}')
        name="oracle-${version}"
        echo $name
        return 0
    fi
    
    #检查是否为 RedHat
    exec_cmd_r "$ipaddr" "120" "ls ${rhel_release}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        version=$(exec_cmd_r "$ipaddr" "120" "cat ${rhel_release}" "2" | grep "^Red Hat Enterprise" | awk '{print $7}')
        name="redhat-${version}"
        echo $name
        return 0
    fi
    
    #检查是否为 SuSE
    exec_cmd_r "$ipaddr" "120" "ls ${sles_release}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        version="$(exec_cmd_r "$ipaddr" "120" "cat ${sles_release}" "2" | grep "^VERSION" | awk '{print $3}')"
        version="${version}"."$(exec_cmd_r "$ipaddr" "120" "cat ${sles_release}" "2" | grep "^PATCHLEVEL" | awk '{print $3}')"
        name="suse-${version}"
        echo $name
        return 0
    fi
    
    #检查是否为 SuSE15
    exec_cmd_r "$ipaddr" "120" "ls ${sles15_release}" "0" > /dev/null
    if [ $? -eq 0 ]; then
        name="$(exec_cmd_r "$ipaddr" "120" "cat ${sles15_release}" "2" | grep "^ID_LIKE" | awk -F "\"" '{print $2}')"
        if [ -z ${name} ];then
            name="$(exec_cmd_r "$ipaddr" "120" "cat ${sles15_release}" "2" | grep "^ID" | awk -F "\"" '{print $2}')"
        fi
        idLike=$(grep "^ID_LIKE=" "${sles15_release}" | awk -F "\"" '{print $2}')
        ID=$(grep "^ID=" "/etc/os-release" | awk -F "\"" '{print $2}')
        VERSION_ID=$(grep "^VERSION_ID=" "/etc/os-release" | awk -F "\"" '{print $2}')
        if [ "${ID}" = "kylin" ] && [ "${idLike}" = "fedora" ]; then
            name="$ID-${VERSION_ID}-fedora"
            return 0
        fi
        version="$(exec_cmd_r "$ipaddr" "120" "cat ${sles15_release}" "2" | grep "^VERSION_ID=" | awk -F "\"" '{print $2}')"
        name="${name}-${version}"
        echo $name
        return 0
    fi
    
    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 1
}

###################################################
#
# 获取OS的名称
#
###################################################
function get_osname()
{
    local support="$1"
    local name=""

    debug "[${support}] Enter [${FUNCNAME}:${LINENO}]"

    if [ -z "$support" ]; then
        error "[${FUNCNAME}:${LINENO}]" "support is null"
        return 1
    fi

    name=$(_get_osname)
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Don't support [$(cat /proc/version)]"
        return 1
    fi

    debug "_get_osname [${name}] [${FUNCNAME}:${LINENO}]"

    #判断是否支持
    echo "$support" | grep -E '^'${name}';|;'${name}';|;'${name}'$' > /dev/null
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Don't support [${name}]. Currently only support [${support}]"
        return 1
    fi

    debug "[${name}] Leave [${FUNCNAME}:${LINENO}]"

    echo "$name"

    return 0
}


###################################################
#
# 根据OS名称获取对应的rpm包目录
#
###################################################
function get_pkg_dir_by_name()
{
    local name="$1"
    local line=""
    local type=""
    local pkg_dir=""

    debug "[${name}]Enter [${FUNCNAME}:${LINENO}]"
    
    if [ -z "$name" ]; then
        error "[${FUNCNAME}:${LINENO}]" "name is null"
        return 1
    fi

    #逐项匹配
    while read line
    do
        if [ -z "$line" ]; then
            continue
        fi

        #获取对应的目录
        type="$(echo $line | awk -F':' '{print $1}')"
        if [ "${type}" == "${name}" ]; then
            pkg_dir="$(echo $line | awk -F':' '{print $2}')"
        fi

        debug "g_pkgs_dir:${line}:${type}:${name}:${pkg_dir}"

    done < <(echo "$g_pkgs_dir" | tr ';' '\n')

    if [ -z "${pkg_dir}" ]; then
        error "[${FUNCNAME}:${LINENO}]" "No local os pkgs dir."
        return 1
    fi

    debug "[${name}:${pkg_dir}]Leave [${FUNCNAME}:${LINENO}]"

    echo "$pkg_dir"

    return 0
}
