#!/bin/bash

###################################################
# 功能: 对各个节点的OS进行适配
###################################################

###################################################
# 失败时返回值范围: 1, 10-100
# 1  :   普通失败
# 10-40: 表示检查失败
# 10 :   python version is too low
# 11 :   os version not supported
# 12 :   diskmgt service has been installed
# 13 :
#
# 50 :   Setup os failed
# 60 :   Security reinforcement failed
# 70 :   Disk format failed
# 
###################################################

###################################################
# 警告: 101-120
# 101  : swap is on, please close swap.
# 
###################################################

###################################################
# 全局变量
###################################################
g_GENERAL_ERROR=1
g_PYTHON_VERSION_LOW=10
g_OS_VERSION_NOT_SUPPORTED=11
g_OPTIMIZE_FAILED=20
g_OPTIMIZE_DIR_NOT_FIND=21
g_SET_OS_FAILED=50
g_SECURITY_FAILED=60
g_SET_RANDOM_DEVICE_FAILED=65
g_DISK_FORMAT_FAILED=70

###################################################
#
# 根据返回码 回显日志
#
###################################################
function show_preinstall_result()
{
    local ret=$1
    local ip=$2
    
    case "$ret" in    
     "$g_GENERAL_ERROR")  
        error "[${FUNCNAME}:${LINENO}]" "[${ip}] general error" 
        return 1
     ;;
     "$g_PYTHON_VERSION_LOW")  
        error "[${FUNCNAME}:${LINENO}]" "[${ip}] python version is too low. The minimum version is 3.0.0" 
        return 1
     ;; 
     
     "$g_OS_VERSION_NOT_SUPPORTED")  
        error "[${FUNCNAME}:${LINENO}]" "[${ip}] os version not supported" 
        return 1
     ;;
     
     "$g_OPTIMIZE_FAILED")  
        error "[${FUNCNAME}:${LINENO}]" "[${ip}] os optimize failed" 
        return 1
     ;;

    "$g_OPTIMIZE_DIR_NOT_FIND")  
        error "[${FUNCNAME}:${LINENO}]" "[${ip}] os_optimize script not found" 
        return 1
     ;;
     
     "$g_SET_OS_FAILED")
        error "[${FUNCNAME}:${LINENO}]" "[${ip}] Setup os failed" 
        return 1
     ;;
     
     "$g_SECURITY_FAILED")
        error "[${FUNCNAME}:${LINENO}]" "[${ip}] Security reinforcement failed" 
        return 1
     ;;
     
     "$g_SET_RANDOM_DEVICE_FAILED")
        error "[${FUNCNAME}:${LINENO}]" "[${ip}] Set feed random data failed" 
        return 1
     ;;
     
     "$g_DISK_FORMAT_FAILED")
        error "[${FUNCNAME}:${LINENO}]" "[${ip}] Disk format failed" 
        return 1
     ;;     
    esac 
    
    error "[${FUNCNAME}:${LINENO}]" "[${ip}] Unknown Error" 
    return 1
}

#系统启用了swap分区
g_SWAP_IS_ON=101

#重复安装
g_DISKMGT_INSTALLED=102

###################################################
#
# 根据返回码 回显警告信息
#
###################################################
function show_warn_result()
{
    local ret=$1
    local ip=$2
    
    case "$ret" in    
     "$g_SWAP_IS_ON")  
        warn "[${FUNCNAME}:${LINENO}]" "[${ip}] swap is on, please close swap." 
        return 0
     ;;
     
     "$g_DISKMGT_INSTALLED")
        warn "[${FUNCNAME}:${LINENO}]" "[${ip}] Diskmgt service has been installed, preinstall may have been executed"
        return 0
     ;;
    esac 
    
    warn "[${FUNCNAME}:${LINENO}]" "[${ip}] Unknown Warn" 
    
    return 0
}

###################################################
#
# 适配各节点
#
###################################################
function preinstall_node()
{
    local ipaddr="$1"     #主机名或ip地址
    local index="$2"      #数组索引
    local parted_conf=""  #节点对应的格式化配置
    local services_ip=""  #节点对应的services IP地址
    local hostname=""     #节点对应的主机名
    local ret=0
    
    debug "[${ipaddr}:${index}]Enter [${FUNCNAME}:${LINENO}]"

    #入参检查
    if [ -z "$ipaddr" ] || [ -z "$index" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Parameter error."
        return 1
    fi
    
    #ping目标端 检测是否可以ping通
    ping -c 1 "$ipaddr" -i 0.2 -W 1 > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        ping6 -c 1 "$ipaddr" -i 0.2 -W 1 > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "ping ${ipaddr} failed."
            set_node_run_failure "${ipaddr}"
            return 1
        fi
    fi

    #登陆到各节点并执行命令
    exec_cmd_r "$ipaddr" "120" "whoami" "0" > /dev/null
    ret=$?
    if [ $ret -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Can not login ${ipaddr}, permission denied. return code:${ret}"
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #在各个节点创建工作空间
    gen_workspace "$g_work_space" "${ipaddr}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Gen workspace failed on ${ipaddr}"
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #设置进度
    set_node_schedule "${ipaddr}" "1"
    
    #检查远端服务器的OS是否匹配当前配置OS
    check_osname_r "${ipaddr}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "os version is inconsistent with g_pkgs_dir=$g_pkgs_dir on ${ipaddr}."
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #在远端服务器安装ssh相关的rpm包
    install_ssh_r "${ipaddr}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Install ssh failed on ${ipaddr}"
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #设置进度
    set_node_schedule "${ipaddr}" "3"

    #在远端服务器安装缺失的rpm包
    add_packages_r "${ipaddr}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Add packages failed on ${ipaddr}"
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    # 建立软链接
    add_libssl_libcrypto_link "${ipaddr}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Add libssl libcrypto or libboost link failed on ${ipaddr}"
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #设置进度
    set_node_schedule "${ipaddr}" "8"
    
    #把适配脚本拷贝至远端服务器
    cp_file_to_r "$ipaddr" "240" "${g_preinstall_path}/modules" "${g_work_space}" "0" > /dev/null
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Copy files to ${ipaddr} failed"
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #设置进度
    set_node_schedule "${ipaddr}" "12"

    #配置core dump
    config_coredump_r "${ipaddr}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "config core dump failed on ${ipaddr}"
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #设置进度
    set_node_schedule "${ipaddr}" "20"

    #配置GaussDB一体机主板刷新crontab
    config_ipmi_r "${ipaddr}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "set ipmi crontab failed on ${ipaddr}"
        set_node_run_failure "${ipaddr}"
        return 1
    fi

    #设置进度
    set_node_schedule "${ipaddr}" "30"

    #获取分区配置
    if [ "${g_parted}" -ne 0 ]; then
        parted_conf="${g_parted_array[${index}]}"
    fi
    
    services_ip="${g_service_ip_array[${index}]}"
    hostname="${g_hostname_array[${index}]}"

    #在各节点上执行适配
    if [ -n "${hostname}" ]; then
        exec_cmd_r "$ipaddr" "18000" "bash ${g_work_space}/modules/install.sh -a \"$g_platform\" -i \"$ipaddr\" -p \"$g_parted\" \
            -c \"$parted_conf\" -d \"$g_debug\" -k \"$g_check\" -o \"$g_support_os\" -w \"$g_swap_off\" \
            -s \"$services_ip\" -n \"$hostname\" -r \"$g_feed_random_setup\" -f \"$g_optimize\" -l \"$g_kms_conf\" -v \"$g_ip_model\" -h \"$g_data_home\""
        ret=$?
    else
        exec_cmd_r "$ipaddr" "18000" "bash ${g_work_space}/modules/install.sh -a \"$g_platform\" -i \"$ipaddr\" -p \"$g_parted\" \
            -c \"$parted_conf\" -d \"$g_debug\" -k \"$g_check\" -o \"$g_support_os\" -w \"$g_swap_off\" -r \"$g_feed_random_setup\" \
            -f \"$g_optimize\" -l \"$g_kms_conf\" -v \"$g_ip_model\" -h \"$g_data_home\""
        ret=$?
    fi
    
    #返回值 1-100 为失败
    if [ $ret -ge 1 -a $ret -le 100 ] || [ $ret -ge 120 ]; then
        show_preinstall_result "${ret}" "${ipaddr}"
        set_node_run_failure "${ipaddr}"
        return $ret
    fi
    
    #返回值 101-120 为告警
    if [ $ret -ge 101 -a $ret -le 120 ]; then
        show_warn_result "${ret}" "${ipaddr}"
    fi

    

    #删除节点上的工作空间
    del_workspace "$g_work_space" "${ipaddr}" > /dev/null 2>&1

    #设置进度为
    set_node_schedule "${ipaddr}" "100"

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}


###################################################
#
# 启动OS适配: 共分为6个步骤
#  1:在各节点创建工作目录        /tmp/fi-preinstall
#  1:把适配脚本拷贝至各节点     拷贝至 /tmp/fi-preinstall
#  2:在各节点调用启动适配脚本   /tmp/fi-preinstall/modules/install.sh
#
###################################################
function start_preinstall()
{
    local i=""       #循环变量
    local ipaddr=""

    debug "Enter [${FUNCNAME}:${LINENO}]"
    
    Nproc=200    # 可同时运行的最大作业数
    PID=() # 记录PID到数组, 检查PID是否存在以确定是否运行完毕
    
    #初始化进程数组
    init_processor_array "${#g_ipaddr_array[@]}"

    for ((i=0;i<${#g_ipaddr_array[@]};i++))
    do
        ipaddr="${g_ipaddr_array[${i}]}"

        #控制并发,检查PID是否存在以确定是否运行完毕,找到一个就退出 
        while true
        do
            for((Ijob=0; Ijob<Nproc; Ijob++))
            do
                if [[ ! "${PID[Ijob]}" ]] || ! kill -0 ${PID[Ijob]} 2> /dev/null
                then
                    break
                fi
            done
            
            if [[ "$Ijob" -lt "$Nproc" ]]
            then 
                break
            fi
            
            sleep 2
        done
        
        #并行对各个节点进行适配
        preinstall_node "${ipaddr}" "${i}" &
        g_processor_array[${i}]=$!
        PID[Ijob]=$!
    done

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}
