#!/bin/bash

#新的秘钥文件
NEW_KEY=""
#磁盘管理服务安装路径
LUKSMGTPATH="/usr/local/kmsLuksMgt"

#连接kms服务配置文件
KMS_CONF_FILE="${LUKSMGTPATH}/ini/kms_info"
KMS_CONF_FILE_BAK="${LUKSMGTPATH}/ini/kms_info.bak"

#kmc工具路径
kmc_tool_dir="${LUKSMGTPATH}/kmc"

#加密分区列表
ENCRYPT_MAPPER_LIST="${LUKSMGTPATH}/ini/kms_encrypt_devs"
TMP_ENCRYPT_MAPPER_LIST="${LUKSMGTPATH}/ini/encrypt_mapper.tmp"

#已经成功替换秘钥的加密分区列表，用于失败时回滚
ROLLBACK_LIST="${LUKSMGTPATH}/ini/rollback.lst"
ROLLBACK_FAIL_LIST="${LUKSMGTPATH}/ini/rollback_fail.lst"

#日志文件
LOGPATH="/var/log/kmsLuksMgt/"
LOGFILE="/var/log/kmsLuksMgt/replace_key.log"

#新的密钥
NEW_KEY_VERSION_NAME="$1"
#旧的秘钥
OLD_KEY_VERSION_NAME=

###################################################
# 显示帮助
###################################################
function show_help()
{
    echo "Usage: $0 1 new version name"
}



########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme()
{
    local log_mode=$1
    local log_info="$2"
    
    
    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi
    
    #root安装，所以没有路径要实时创建
    mkdir -p "${LOGPATH}"
    if [ $? -ne 0 ]; then
        echo "[$(date)] [EMER] mkdir -p ${LOGPATH} failed."
        echo "[$(date)] ${log_info}"
        return 1
    fi
    
    chmod 700 "${LOGPATH}"
    
    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)] ${log_info}" | tee -a "${LOGFILE}"  #通过tee追加显示
    else
        echo "logme invalid mode para."
        return 1
    fi
    
    return 0
}

function init()
{
    
    kms_server=$(cat ${KMS_CONF_FILE} | grep "^[[:space:]]*kms_server=" | awk '{print $1}' | sed 's/kms_server=//g' | head -n 1 2>/dev/null)
    if [ -z "$kms_server" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get kms server from $KMS_CONF_FILE."
        return 1
    fi
    
    OLD_KEY_VERSION_NAME=$(cat ${KMS_CONF_FILE} | grep "^[[:space:]]*kms_key_version_name=" | awk '{print $1}' | sed 's/kms_key_version_name=//g' | head -n 1 2>/dev/null)
    if [ -z "$OLD_KEY_VERSION_NAME" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get kms key version name from $KMS_CONF_FILE."
        return 1
    fi
    
    rm -f "${TMP_ENCRYPT_MAPPER_LIST}"
    cat "${ENCRYPT_MAPPER_LIST}" > "${TMP_ENCRYPT_MAPPER_LIST}"
    if [ $? -ne 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Failed to cat ${ENCRYPT_MAPPER_LIST} to ${TMP_ENCRYPT_MAPPER_LIST}."
        return 1
    fi
}

###################################################
# 参数检查
###################################################
function chk_params()
{
    params=$1
    cd $kmc_tool_dir > /dev/null 2>&1
    ./kmc_tool -g -i "GET https://${kms_server}/kms/v1/keyversion/${params}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${NEW_KEY_VERSION_NAME} is not in kms service."
    fi
    cd - > /dev/null 2>&1
    
}

###################################################
# 备份旧信息配置
###################################################
function backup()
{
    if [ -f ${KMS_CONF_FILE_BAK} ]; then
        rm -rf ${KMS_CONF_FILE_BAK}
    fi
    
    cp -pf ${KMS_CONF_FILE} ${KMS_CONF_FILE_BAK}
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to backup ${KMS_CONF_FILE}."
    fi
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Backup ${KMS_CONF_FILE} to ${KMS_CONF_FILE_BAK}."
    return 0
}

function getMaterial()
{
    local kms_server=$1
    local kms_key_version=$2
    local kmc_tool_dir=$3
    cd $kmc_tool_dir > /dev/null 2>&1
    material=$(./kmc_tool -g -i "GET https://${kms_server}/kms/v1/keyversion/${kms_key_version}")
    cd - > /dev/null 2>&1
    if [ -z "${material}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get Material from ${kms_server}."
        return 1
    fi
    echo $material
    return 0
}

###################################################
# 为单个加密分区添加新秘钥
###################################################
function add_new_key()
{
    local mapper="$1"
    local dev="$2"
    local new_key="$3"
    local old_key="$4"
    
    #获取旧密钥
    old_material=$(getMaterial ${kms_server} ${old_key} ${kmc_tool_dir})
    
    
    #获取新密钥
    new_material=$(getMaterial ${kms_server} ${new_key} ${kmc_tool_dir})
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Start to add key $new_key for $mapper."    
    #添加新秘钥
    echo "[INFO] [${FUNCNAME}:${LINENO}] Start mapper encrypted $partation to $dev_mapper."
expect <<EOF
set timeout 30;
spawn -noecho cryptsetup luksAddKey "$dev"
expect   "passphrase"
send "${old_material}\r"
expect "new passphrase"
send "${new_material}\r"
expect "Verify"
send "${new_material}\r"
expect eof
EOF
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to add key $new_key for $mapper($dev) with ${NEW_KEY_VERSION_NAME}."
        return 1
    fi
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Success to add key $NEW_KEY_VERSION_NAME for $mapper($dev)."
    return 0
}

###################################################
# 删除旧秘钥
###################################################
function delete_old_key()
{
    local mapper="$1"
    local dev="$2"
    local old_key="$3"
    
    #获取旧密钥
    old_material=$(getMaterial ${kms_server} ${old_key} ${kmc_tool_dir})
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Start to delete key $OLD_KEY_VERSION_NAME for $mapper."
    #删除旧秘钥
expect <<EOF
set timeout 30;
spawn -noecho cryptsetup luksRemoveKey "$dev"
expect   "passphrase"
send "${old_material}\r"
expect eof
EOF
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to delete key $OLD_KEY_VERSION_NAME for $mapper($dev)."
        return 1
    fi
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Success to delete key $OLD_KEY_VERSION_NAME for $mapper($dev)."
    return 0
}

###################################################
# 替换秘钥
###################################################
function replace_key()
{
    local mapper="$1"
    local dev=""
    local old_key="$2"
    local new_key="$3"
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Start to replace key for $mapper."
    if [ ! -b "/dev/mapper/$mapper" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] /dev/mapper/$mapper is not a block device."
        return 1
    fi
       
    cryptsetup status "$mapper" >> "${LOGFILE}" 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get status for $mapper."
        return 1
    fi
    
    dev=$(cryptsetup status "$mapper" | grep 'device:' | awk '{print $2}')
    if [ -z "$dev" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get device for $mapper."
    elif [ ! -b "$dev" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] $dev is not a block device."
    fi
    
    add_new_key "$mapper" "$dev" "$new_key" "$old_key"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to add key $NEW_KEY_VERSION_NAME for $mapper($dev)."
        return 1
    fi
    
    delete_old_key "$mapper" "$dev" "$old_key"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to delete key $OLD_KEY_VERSION_NAME for $mapper($dev)."
        return 1
    fi
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Success to replace key for $mapper."
    return 0
}

###################################################
# 替换秘钥
###################################################
replace_key_for_all()
{
    local old_key="$1"
    local new_key="$2"
    local line="" 
    local ret=""
    
    if [ ! -s "$ENCRYPT_MAPPER_LIST" ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] No encrypt device."
        return 0
    fi
    
    rm -f "${ROLLBACK_LIST}"
    #依次读加密设备列表的每一行进行处理
    while read line
    do
        echo ${line} | grep -e "^UUID="
        if [ $? -ne 0 ]; then
            continue
        fi
        
        #如果上次回滚失败，则说明上次新添加的访问秘钥未成功清除，本次重入可以跳过
        if [ -f "$ROLLBACK_FAIL_LIST" ]; then
            grep -wq "${line}" "$ROLLBACK_FAIL_LIST" >> "${LOGFILE}" 2>&1
            if [ $? -eq 0 ]; then
                logme 2 "[WARN] [${FUNCNAME}:${LINENO}] Find ${line} in $ROLLBACK_FAIL_LIST. So skip to replace it this time."
                continue
            fi
        fi
        line=$(echo ${line} | awk '{print $2}')
        replace_key "${line}" "${old_key}" "${new_key}"
        ret=$?
        if [ "$ret" -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to replace key for line(${line})."            
            break
        else
            #记录已经替换成功的加密分区，用于失败后回滚
            echo "${line}" >> "${ROLLBACK_LIST}"
        fi
    done <"${ENCRYPT_MAPPER_LIST}"
    
    return "$ret"
}

###################################################
# 回滚已经替换成功的秘钥
###################################################
rollback()
{
    local old_key="$1"
    local new_key="$2"
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Begin to rollback."
    if [ ! -s "$ROLLBACK_LIST" ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] No device to rollback."
        return 0
    fi
    
    rm -f "${ROLLBACK_FAIL_LIST}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to rm -f ${ROLLBACK_FAIL_LIST}."
        return 1
    fi
    
    local ret=0
    #依次对已成功替换的加密设备进行回滚
    while read line
    do
        replace_key "${line}" "${new_key}" "${old_key}"
        if [ "$?" -ne 0 ]; then
            ret=1
            #记录已回滚失败的加密分区，下次重入时跳过
            echo "${line}" >> "${ROLLBACK_FAIL_LIST}"
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to rollback key for line(${line})."            
        else
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Success to rollback key for line(${line})."
        fi
    done <"${ROLLBACK_LIST}"
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] End to rollback."
    return "$ret"
}

###################################################
# 更新update_kms_info文件中的密钥版本
###################################################
function update_kms_info()
{
    KMS_CONF_FILE_TMP="${KMS_CONF_FILE}.tmp"
    
    cp -pf "$KMS_CONF_FILE" "$KMS_CONF_FILE_TMP"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to backup ${KMS_CONF_FILE}."
        return 1
    fi
    
        #如果替换的秘钥为当前秘钥，将备用秘钥更换为当前秘钥
        sed -i "s#${OLD_KEY_VERSION_NAME}#${NEW_KEY_VERSION_NAME}#g" "$KMS_CONF_FILE_TMP"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to replace ${OLD_KEY} with ${KEY_IN_USE} for ${crypttab_file}."
            return 1
        fi

    
    
    mv -f "$KMS_CONF_FILE_TMP" "$KMS_CONF_FILE"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to mv -f $crypttab_tmp $crypttab_file."
        return 1
    fi
    
    return 0
}

###################################################
# 主函数
###################################################
function main()
{
    #初始化配置信息
    init
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to init."
        return 1
    fi
    
    #参数检查
    chk_params "$1"
    if [ $? -ne 0 ]; then
        show_help
        return 1
    fi
    
    #备份密钥
    backup
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to backup."
        return 1
    fi
    
    #为所有加密分区替换秘钥
    replace_key_for_all "$OLD_KEY_VERSION_NAME" "$NEW_KEY_VERSION_NAME"
    if [ $? -ne 0 ]; then
        rollback "$OLD_KEY_VERSION_NAME" "$NEW_KEY_VERSION_NAME"
        return 1
    fi
    
    #将新更换的秘钥设置成当前秘钥
    update_kms_info "new_key"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to update new_key to /etc/crypttab."
        return 1
    fi
    
    rm -f "${OLD_KEY_BACKUP}"
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Success to replace $OLD_KEY with $NEW_KEY for all encrypted device." 
    return 0
}


if [ ! -d "${LUKSMGTPATH}" ]; then
    logme 2 "[WARN] [${FUNCNAME}:${LINENO}] Path ${LUKSMGTPATH} doesn't exist. Skip replace key for this node!"
    exit 0
fi

main "$@"
exit $?
