#!/bin/bash

#待替换的秘钥编号
KEY_NUMBER=""
#新的秘钥文件
NEW_KEY=""
ORIGINAL_NEW_KEY=""
#磁盘管理服务安装路径
DISKMGTPATH="/usr/local/diskmgt"
#秘钥文件路径
KEY_PATH="${DISKMGTPATH}/ini-plugin/keys"
#秘钥文件
KEY_1="${DISKMGTPATH}/ini-plugin/keys/fi_disk_partition_key_1"
KEY_2="${DISKMGTPATH}/ini-plugin/keys/fi_disk_partition_key_2"
#旧的秘钥文件
OLD_KEY=""
OLD_KEY_BACKUP=""
#当前/etc/crypttab中使用的秘钥文件
KEY_IN_USE=""
#加密分区列表
ENCRYPT_MAPPER_LIST="${DISKMGTPATH}/ini-plugin/keys/encrypt_mapper.lst"
TMP_ENCRYPT_MAPPER_LIST="${DISKMGTPATH}/ini-plugin/keys/encrypt_mapper.tmp"
#已经成功替换秘钥的加密分区列表，用于失败时回滚
ROLLBACK_LIST="${DISKMGTPATH}/ini-plugin/keys/rollback.lst"
ROLLBACK_FAIL_LIST="${DISKMGTPATH}/ini-plugin/keys/rollback_fail.lst"
#日志文件
LOGPATH="/var/log/diskmgt/"
LOGFILE="/var/log/diskmgt/replace_key.log"
CURRENT_PATH=""
SCC_CONF="${DISKMGTPATH}/scc/config/scc.conf"
PLAIN_PWD_FILE_SUFFIX="plain"

###################################################
# 显示帮助
###################################################
function show_help() {
    echo "Usage:"
    echo "      $0 1 [/usr/local/diskmgt/ini-plugin/keys/new_key]"
    echo "      $0 2 [/usr/local/diskmgt/ini-plugin/keys/new_key]"
}

########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme() {
    local log_mode=$1
    local log_info="$2"

    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi

    #root安装，所以没有路径要实时创建
    mkdir -p "${LOGPATH}"
    if [ $? -ne 0 ]; then
        echo "[$(date)] [EMER] diskmgtmodule ins: mkdir -p ${LOGPATH} failed."
        echo "[$(date)] ${log_info}"
        return 1
    fi

    chmod 700 "${LOGPATH}"

    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)] ${log_info}" | tee -a "${LOGFILE}" #通过tee追加显示
    else
        echo "logme invalid mode para."
        return 1
    fi

    return 0
}

################################################################################
# Function: get_current_path
# Description: 获取脚本所在的目录及脚本名
################################################################################
get_current_path() {
    cd "$(dirname "${BASH_SOURCE-$0}")"
    local currentPath="${PWD}"
    cd - >/dev/null
    echo "${currentPath}"
}

###################################################
# 初始化新秘钥
###################################################
function init_new_key() {
    NEW_KEY="${KEY_PATH}/new_key"
    local newKeyPlainText=$(generate_random_pwd)
    echo "${newKeyPlainText}" >"${NEW_KEY}"
}

###################################################
# 获取秘钥文件(解密后)名称
###################################################
function build_key_plain_text_file_name() {
    local keyFile="$1"
    echo "${keyFile}.${PLAIN_PWD_FILE_SUFFIX}"
}

###################################################
# 获取解密后的秘钥
###################################################
function get_key_plain_text() {
    local keyFile="$1"
    local keyPlainTextFile="$2"
    rm -rf "${keyPlainTextFile}"

    if [ ! -s "${keyFile}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] The key file[${keyFile}] is empty or it doesn't exist"
        return 1
    fi

    local keyPlainText=$(decrypt_password $(cat "${keyFile}") "${SCC_CONF}")
    if [ $? -ne 0 ] || [ -z ${keyPlainText} ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed decrypt password with key_file[${keyFile}]"
        return 1
    fi

    echo "${keyPlainText}" >"${keyPlainTextFile}"
    return 0
}

###################################################
# 参数检查
###################################################
function chk_params() {
    if [ $# -eq 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] The number of parameters is incorrect. At least one parameter must be contained."
        return 1
    fi

    KEY_NUMBER="$1"
    NEW_KEY="$2"

    if [ "$KEY_NUMBER" -ne 1 -a "$KEY_NUMBER" -ne 2 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Value of first para must be 1 or 2."
        return 1
    fi

    if [ ! -s "$NEW_KEY" ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] The key file $NEW_KEY is empty or it doesn't exist, so use a random key."
        init_new_key
    fi

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Start to replace key $KEY_NUMBER with new key file $NEW_KEY."
    return 0
}

###################################################
# 备份旧秘钥
###################################################
function init() {
    if [ "${KEY_NUMBER}" -eq 1 ]; then
        OLD_KEY="$KEY_1"
        KEY_IN_USE="$KEY_2"
    elif [ "${KEY_NUMBER}" -eq 2 ]; then
        OLD_KEY="$KEY_2"
        KEY_IN_USE="$KEY_1"
    else
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Invalid key number $KEY_NUMBER."
        return 1
    fi
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Key $OLD_KEY will be replaced."

    rm -f "${TMP_ENCRYPT_MAPPER_LIST}"
    cat "${ENCRYPT_MAPPER_LIST}" >"${TMP_ENCRYPT_MAPPER_LIST}"
    if [ $? -ne 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Failed to cat ${ENCRYPT_MAPPER_LIST} to ${TMP_ENCRYPT_MAPPER_LIST}."
        return 1
    fi

    chmod 600 "${NEW_KEY}"
    if [ $? -ne 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Failed to chmod 600 ${NEW_KEY}."
        return 1
    fi

    return 0
}

###################################################
# 加密新秘钥
###################################################
function encrypt_new_key() {
    ORIGINAL_NEW_KEY="${NEW_KEY}"
    local newKeyCipher=$(encrypt_password "$(cat $NEW_KEY)" "${SCC_CONF}")
    local newKeyCipherFile="${NEW_KEY}.cipher"
    echo "${newKeyCipher}" >"${newKeyCipherFile}"
    NEW_KEY="${newKeyCipherFile}"
}

###################################################
# 备份旧秘钥
###################################################
function backup() {
    if [ ! -s "${OLD_KEY}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Key file ${KEY_1} doesn't exist."
        return 1
    fi

    OLD_KEY_BACKUP="${OLD_KEY}_bak"
    cp -pf "${OLD_KEY}" "${OLD_KEY_BACKUP}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to backup ${OLD_KEY}."
        return 1
    fi

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Backup ${OLD_KEY} to $OLD_KEY_BACKUP."
    return 0
}

###################################################
# 更新/etc/crypttab文件中加密映射的keyfile
###################################################
function update_crypttab() {
    local crypttab_file="/etc/crypttab"
    local crypttab_tmp="/etc/crypttab.fi.tmp"

    cp -pf "$crypttab_file" "$crypttab_tmp"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to backup ${crypttab_file}."
        return 1
    fi
    if [ "$1" == "backup_key" ]; then
        #如果替换的秘钥为当前秘钥，将备用秘钥更换为当前秘钥
        sed -i "s#${OLD_KEY}#${KEY_IN_USE}#g" "$crypttab_tmp"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to replace ${OLD_KEY} with ${KEY_IN_USE} for ${crypttab_file}."
            return 1
        fi
    elif [ "$1" == "new_key" ]; then
        #将替换后的秘钥更换为当前秘钥
        sed -i "s#${KEY_IN_USE}#${OLD_KEY}#g" "$crypttab_tmp"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to replace ${KEY_IN_USE} with ${OLD_KEY} for ${crypttab_file}."
            return 1
        fi
    else
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Argument is error"
        return 1
    fi

    mv -f "$crypttab_tmp" "$crypttab_file"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to mv -f $crypttab_tmp $crypttab_file."
        return 1
    fi

    return 0
}

###################################################
# 为单个加密分区添加新秘钥
###################################################
function add_new_key() {
    local mapper="$1"
    local dev="$2"
    local new_key="$3"

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Start to add key $new_key for $mapper."

    local newKeyPlainTextFile=$(build_key_plain_text_file_name "${new_key}")
    get_key_plain_text "${new_key}" "${newKeyPlainTextFile}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get key plain text from key-file[${new_key}]."
        return 1
    fi

    local keyInUsePlainTextFile=$(build_key_plain_text_file_name "${KEY_IN_USE}")
    get_key_plain_text "${KEY_IN_USE}" "${keyInUsePlainTextFile}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get key plain text from key-file[${KEY_IN_USE}]."
        return 1
    fi

    #添加新秘钥
    cryptsetup --key-file "${keyInUsePlainTextFile}" luksAddKey "$dev" "$newKeyPlainTextFile" >>"${LOGFILE}" 2>&1
    local retVal=$?

    rm -rf "${newKeyPlainTextFile}"
    rm -rf "${keyInUsePlainTextFile}"
    if [ ${retVal} -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to add key $new_key for $mapper($dev) with ${KEY_IN_USE}."
        return 1
    fi

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Success to add key $new_key for $mapper($dev)."
    return 0
}

###################################################
# 删除旧秘钥
###################################################
function delete_old_key() {
    local mapper="$1"
    local dev="$2"
    local old_key="$3"

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Start to delete key $old_key for $mapper."
    local oldKeyPlainTextFile=$(build_key_plain_text_file_name "${old_key}")
    get_key_plain_text "${old_key}" "${oldKeyPlainTextFile}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get key plain text from key-file[${old_key}]."
        return 1
    fi

    #删除旧秘钥
    cryptsetup luksRemoveKey "$dev" "$oldKeyPlainTextFile" >>"${LOGFILE}" 2>&1
    local retVal=$?

    rm -rf "${oldKeyPlainTextFile}"
    if [ ${retVal} -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to delete key $old_key for $mapper($dev)."
        return 1
    fi

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Success to delete key $old_key for $mapper($dev)."
    return 0
}

###################################################
# 替换秘钥
###################################################
function replace_key() {
    local mapper="$1"
    local old_key="$2"
    local new_key="$3"
    local dev=""

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Start to replace key for $mapper."
    if [ ! -b "/dev/mapper/$mapper" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] /dev/mapper/$mapper is not a block device."
        return 1
    fi

    cryptsetup status "$mapper" >>"${LOGFILE}" 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get status for $mapper."
        return 1
    fi

    dev=$(cryptsetup status "$mapper" | grep 'device:' | awk '{print $2}')
    if [ -z "$dev" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get device for $mapper."
    elif [ ! -b "$dev" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] $dev is not a block device."
    fi

    add_new_key "$mapper" "$dev" "$new_key"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to add key $new_key for $mapper($dev)."
        return 1
    fi

    delete_old_key "$mapper" "$dev" "$old_key"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to delete key $old_key for $mapper($dev)."
        return 1
    fi

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Success to replace key for $mapper."
    return 0
}

###################################################
# 替换秘钥
###################################################
replace_key_for_all() {
    local old_key="$1"
    local new_key="$2"
    local line=""
    local ret=""

    if [ ! -s "$TMP_ENCRYPT_MAPPER_LIST" ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] No encrypt device."
        return 0
    fi

    rm -f "${ROLLBACK_LIST}"
    #依次读加密设备列表的每一行进行处理
    while read line; do
        #如果上次回滚失败，则说明上次新添加的访问秘钥未成功清除，本次重入可以跳过
        if [ -f "$ROLLBACK_FAIL_LIST" ]; then
            grep -wq "${line}" "$ROLLBACK_FAIL_LIST" >>"${LOGFILE}" 2>&1
            if [ $? -eq 0 ]; then
                logme 2 "[WARN] [${FUNCNAME}:${LINENO}] Find ${line} in $ROLLBACK_FAIL_LIST. So skip to replace it this time."
                continue
            fi
        fi

        replace_key "${line}" "${old_key}" "${new_key}"
        ret=$?
        if [ "$ret" -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to replace key for line(${line})."
            break
        else
            #记录已经替换成功的加密分区，用于失败后回滚
            echo "${line}" >>"${ROLLBACK_LIST}"
        fi
    done <"${TMP_ENCRYPT_MAPPER_LIST}"

    return "$ret"
}

###################################################
# 回滚已经替换成功的秘钥
###################################################
rollback() {
    local old_key="$1"
    local new_key="$2"

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Begin to rollback."
    if [ ! -s "$ROLLBACK_LIST" ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] No device to rollback."
        return 0
    fi

    rm -f "${ROLLBACK_FAIL_LIST}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to rm -f ${ROLLBACK_FAIL_LIST}."
        return 1
    fi

    local ret=0
    #依次对已成功替换的加密设备进行回滚
    while read line; do
        replace_key "${line}" "${new_key}" "${old_key}"
        if [ "$?" -ne 0 ]; then
            ret=1
            #记录已回滚失败的加密分区，下次重入时跳过
            echo "${line}" >>"${ROLLBACK_FAIL_LIST}"
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to rollback key for line(${line})."
        else
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Success to rollback key for line(${line})."
        fi
    done <"${ROLLBACK_LIST}"

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] End to rollback."
    return "$ret"
}

function post_replace_key() {
    rm -rf "${OLD_KEY_BACKUP}"
    rm -rf "${ORIGINAL_NEW_KEY}"
    rm -rf "${ROLLBACK_LIST}"
    rm -rf "${TMP_ENCRYPT_MAPPER_LIST}"
    chmod 400 ${KEY_PATH}/fi_*
}

###################################################
# 主函数
###################################################
function main() {
    #参数检查
    chk_params "$@"
    if [ $? -ne 0 ]; then
        show_help
        return 1
    fi

    init
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to init."
        return 1
    fi

    encrypt_new_key
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to encrypt new_key."
        return 1
    fi

    backup
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to backup."
        return 1
    fi

    #替换秘钥之前先把/etc/crypttab更新为备用秘钥，避免替换失败后影响分区的开机自动挂载
    update_crypttab "backup_key"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to update backup_key to /etc/crypttab ."
        return 1
    fi

    #为所有加密分区替换秘钥
    replace_key_for_all "$OLD_KEY" "$NEW_KEY"
    if [ $? -ne 0 ]; then
        rollback "$OLD_KEY" "$NEW_KEY"
        return 1
    fi

    mv -f "${NEW_KEY}" "${OLD_KEY}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to mv ${NEW_KEY} -f ${OLD_KEY}."
        return 1
    fi

    #将新更换的秘钥设置成当前秘钥
    update_crypttab "new_key"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to update new_key to /etc/crypttab."
        return 1
    fi

    post_replace_key

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Success to replace $OLD_KEY with $NEW_KEY for all encrypted device."
    return 0
}

if [ ! -d "${DISKMGTPATH}" ]; then
    logme 2 "[WARN] [${FUNCNAME}:${LINENO}] Path ${DISKMGTPATH} doesn't exist. Skip replace key for this node!"
    exit 0
fi

CURRENT_PATH=$(get_current_path)
source ${CURRENT_PATH}/scc_tool.sh

main "$@"
exit $?
