#!/bin/bash

###################################################
# Usage: version.sh
# function: register and check setup tool version
###################################################

declare g_cur_path=""                                # script path
declare g_script_name=""                             # script name
declare g_setup_tool_package_home=""                 # Setup Tool package home directory


###################################################
#
# record audit log
#
###################################################
function syslog()
{
    local component=$1
    local filename=$2
    local status=$3
    local message=$4

    which logger >/dev/null 2>&1
    [ "$?" -ne "0" ] && return 0;

    login_user_ip="$(who -m | sed 's/.*(//g;s/)*$//g')"
    execute_user_name="$(whoami)"
    logger -p local0.notice -i "FusionInsight;${component};[${filename}];${status};${login_user_ip};${execute_user_name};${message}"
    return 0
}

################################################################################
# Function: get_cur_path
# Description: Get script path and name
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
get_cur_path()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_cur_path="${PWD}"
    g_script_name="$(basename "${BASH_SOURCE-$0}")"
    cd - >/dev/null
}

################################################################################
# Function: init_global_param
# Description: init golbal paremeters
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
init_global_param()
{
    g_setup_tool_package_home="$(dirname "${g_cur_path}")"
    if [ -z "${SETUPTOOL_HOME_PATH}" ] ; then
        SETUPTOOL_HOME_PATH="${DEFAULT_SETUPTOOL_HOME_PATH}"
        return 1
    fi
}

########################################################################
# Function: register_version
# Description: register setup tool version
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
########################################################################
function register_version()
{

    if [ ! -f "${g_setup_tool_package_home}/release" ] ; then
        echo "Failed to find release file."
        return 1
    fi


    if [ ! -f "${SETUPTOOL_HOME_PATH}" ] ; then
        mkdir -p "${SETUPTOOL_HOME_PATH}" > /dev/null 2>&1
        if [ $? -ne 0 ] ; then           
            echo "Failed to mkdir ${SETUPTOOL_HOME_PATH}"
            return 1
        fi
    fi

    cp -f "${g_setup_tool_package_home}/release" "${SETUPTOOL_HOME_PATH}/release"
    ret=$?
    export SETUPTOOL_HOME_PATH=${SETUPTOOL_HOME_PATH}

    syslog "Setup Tool" "version.sh" "$ret" "Register Setup Tool version."
}

########################################################################
# Function: get_registered_version
# Description: get registered tool version
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
########################################################################
function get_registered_version()
{

    if [ ! -f "${SETUPTOOL_HOME_PATH}/release" ] ; then
        return 1
    fi
    cat ${SETUPTOOL_HOME_PATH}/release | grep "version" | awk -F "=" '{print $2}'
}

########################################################################
# Function: get_version
# Description: get version from release file
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
########################################################################
function get_version()
{
    if [ $# -lt 1 ]; then
        echo "Illegal argument, accept 1 argument at least."
        return 1
    fi
    local release_file=$1
    if [ ! -f "${release_file}" ] ; then
        return 1
    fi
    cat ${release_file} | grep "version" | awk -F "=" '{print $2}'
}

########################################################################
# Function: get_package_version
# Description: get packaged tool version
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
########################################################################
function get_package_version()
{

    if [ ! -f "${g_setup_tool_package_home}/release" ] ; then
        return 1
    fi

    cat ${g_setup_tool_package_home}/release | grep "version" | awk -F "=" '{print $2}'
}

################################################################################
# Function: main
# Description: Entrance of the script
# Parameter:
#   input:
#   $1  function's name & arguments
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
main()
{
    if [ $# -lt 1 ]; then
        echo "Illegal argument, accept 1 argument at least."
        return 1
    fi
    init_global_param
    local function_name=$1
    shift

    $function_name "$@"
}

PATH="$PATH:/sbin/:/usr/sbin/"

get_cur_path
. "${g_cur_path}/constants.sh" || { log_error "${LINENO}" "Failed to load ${g_curPath}/constants.sh."; exit 1; }

umask 027
main "$@"
exit $?