#!/bin/bash
##############################################################################################
## Function Description:
#### This script is used to expand disk partitions in the cloud environment.
## Constraints:
#### 1. Only the root user can run this script.
#### 2. The disk partition format is GPT.
#### 3. A single partition occupies the entire disk space (one disk occupies one partition).
#### 3. Only partitions of data disks can be expanded. System disks are not supported.
#### 4. The disk partition file system is ext.
##############################################################################################

declare LOGPATH="/var/log/diskmgt"
declare LOGFILE="${LOGPATH}/expand_disks_partition_tool.log"
declare g_mountPoint="$1"
declare g_diskSizeExpandThreshold=100  # unit: MB

function log() {
    local log_mode=$1
    local log_info="$2"

    local dateStr="$(date -d today +'%Y-%m-%d %H:%M:%S.%3N')"

    if [ ${log_mode} -eq 0 ]; then
        echo "[${dateStr}] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[${dateStr}] [$$] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[${dateStr}] [$$] ${log_info}" | tee -a "${LOGFILE}" #通过tee追加显示
    else
        echo "log invalid mode parameter."
        return 1
    fi

    return 0
}

function transferByteToMB() {
    local byteVal=$1
    echo "$[byteVal/1024/1024]"
}

function do_expand_disk_part_fs() {
    local devPath="$1"
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] Start to expand disk partition file system size(dev-path:[${devPath}])"
    local preDevPartFsSize="$(df -h | grep -w ${devPath} | awk -F '[ ]+' '{print $2}')"
    local expandDiskPartitionFSCmd="resize2fs ${devPath}"
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] Execute expand-disk-partition-file-system command:${expandDiskPartitionFSCmd}"
    ${expandDiskPartitionFSCmd} >> "${LOGFILE}" 2>&1
    if [ $? -ne 0 ]; then
        log 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to execute expand-disk-partition-file-system command:${expandDiskPartitionFSCmd}"
        return 1
    fi

    local currentDevPartFsSize="$(df -h | grep -w ${devPath} | awk -F '[ ]+' '{print $2}')"
    if [ "${preDevPartFsSize}" != "${currentDevPartFsSize}" ]; then
        log 1 "[INFO] [${FUNCNAME}:${LINENO}] The file system size of the disk partition[${devPath}] has changed[${preDevPartFsSize}->${currentDevPartFsSize}]."
    fi
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] End to expand disk partition file system size(dev-path:[${devPath}])"
}

function do_expand_disk_partition() {
    local diskPath="$1"
    local devPath="$2"
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] Start to expand disk partition(dev-path:[${devPath}], disk-path:[${diskPath}])"

    which growpart > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        log 1 "[ERROR] [${FUNCNAME}:${LINENO}] The growpart expansion tool does not exist."
        return 1
    fi

    local preDevPartSize="$(lsblk -p -o KNAME,NAME,SIZE | grep -w ${devPath} | awk -F '[ ]+' '{print $3}')"
    local devNumber="${devPath#*${diskPath}}"
    local expandDiskPartitionSizeCmd="growpart ${diskPath} ${devNumber}"
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] Execute expand-disk-partition-size command:${expandDiskPartitionSizeCmd}"
    ${expandDiskPartitionSizeCmd} >> "${LOGFILE}" 2>&1
    if [ $? -ne 0 ]; then
        log 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to execute expand-disk-partition-size command:${expandDiskPartitionSizeCmd}"
        return 1
    fi

    local currentDevPartSize="$(lsblk -p -o KNAME,NAME,SIZE | grep -w ${devPath} | awk -F '[ ]+' '{print $3}')"
    if [ "${preDevPartSize}" != "${currentDevPartSize}" ]; then
        log 1 "[INFO] [${FUNCNAME}:${LINENO}] The disk partition[${devPath}] space has changed[${preDevPartSize}->${currentDevPartSize}]."
    fi

    log 1 "[INFO] [${FUNCNAME}:${LINENO}] End to expand disk partition(dev-path:[${devPath}], disk-path:[${diskPath}])."
}

function expand_disk_partition() {
     # obtain partition info
    local partitionInfo="$(lsblk -p -b -o KNAME,NAME,PKNAME,SIZE,MOUNTPOINT | grep -w ${g_mountPoint})"
    local devPath="$(echo ${partitionInfo} | awk -F '[ ]+' '{print $1}')"
    local diskPath="$(echo ${partitionInfo} | awk -F '[ ]+' '{print $3}')"
    local devSize="$(echo ${partitionInfo} | awk -F '[ ]+' '{print $4}')" # unit: B
    devSize="$(transferByteToMB ${devSize})"

    # obtain disk info
    # local diskPath="${pDevName}"
    local diskInfo="$(lsblk -p -b -o KNAME,NAME,SIZE | grep -w ${diskPath})"
    local diskSize="$(echo ${diskInfo} | awk -F '[ ]+' '{print $3}')"  # unit: B
    diskSize="$(transferByteToMB ${diskSize})"
    log 1 "[INFO] [${FUNCNAME}:${LINENO}] mount-point:[${g_mountPoint}], dev-path:[${devPath}], dev-size:[${devSize}MB], disk-path:[${diskPath}], disk-size:[${diskSize}MB]"

    local expandDiskSize=`expr ${diskSize} - ${devSize}`
    if [ ${expandDiskSize} -gt ${g_diskSizeExpandThreshold} ]; then
        do_expand_disk_partition "${diskPath}" "${devPath}"
        if [ $? -ne 0 ]; then
            return 1
        fi
    fi
    do_expand_disk_part_fs "${devPath}"
}

function init() {
    if [ ! -d "${LOGPATH}" ]; then
        mkdir -p "${LOGPATH}"
    fi

    chmod 700 "${LOGPATH}"

    if [ ! -f "${LOGFILE}" ]; then
        touch "${LOGFILE}"
    fi
}

function pre_handle_node() {
    local eulerReleaseFile="/etc/euleros-release"
    local archType="$(arch)"
    if [ ! -f "${eulerReleaseFile}" ] || [ "$(arch)" != "x86_64" ]; then
        return
    fi

    local eulerReleaseVersion="$(cat ${eulerReleaseFile} | tr -d '()' | awk '{print $4}' | tr [A-Z] [a-z])"
     if [ "${eulerReleaseVersion}" != "sp5" ] ;  then
       return
    fi


    rpm -qa | grep libicu-50.1.2-15.h3.eulerosv2r7.x86_64 > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        log 1 "[INFO] [${FUNCNAME}:${LINENO}] Supplement the RPM package(libicu) on linux os (euleros-2.0-sp5-x86_64)"
        rpm -ivh /tmp/libicu-50.1.2-15.h3.eulerosv2r7.x86_64.rpm
    fi

    rpm -qa | grep gdisk-0.8.6-5.eulerosv2r7.x86_64 > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        log 1 "[INFO] [${FUNCNAME}:${LINENO}] Supplement the RPM package(gdisk) on linux os (euleros-2.0-sp5-x86_64)"
        rpm -ivh /tmp/gdisk-0.8.6-5.eulerosv2r7.x86_64.rpm
    fi

}

function main() {
    if [[ $UID -ne 0 ]]; then
        log 0 "[ERROR] Only the root user can run this script."
        exit 1
    fi

    init

    pre_handle_node

    lsblk -o MOUNTPOINT | grep "/srv/BigData/" | sed "/^#/d" | sed "/^$/d" | while read line
    do
        g_mountPoint="${line}"
        expand_disk_partition
        if [ $? -ne 0 ]; then
            log 0 "[ERROR] Failed to expand the disk partition. Please view ${LOGFILE} for detail info on the node[$(hostname -i)]."
        fi
    done
    log 0 "Disk partition expansion is complete."
}

main
exit $?