#!/usr/bin/python

import ctypes
from AlgorithmID import AlgorithmID
from ErrorCode import ErrorCode
from CryptoBase import Crypto
from CryptoBase import SEC_SUCCESS
from CryptoBase import SEC_TRUE
from CryptoType import SccOptions
from CryptoType import sig_str_t
from CryptoType import kmc_sig_params_t
from CryptoType import configuration


class CryptoAPI(Crypto):
    def __init__(self):
        super(CryptoAPI, self).__init__()

    def initConfig(self, config):
        conf = configuration()
        conf.primaryKeyStoreFile = ctypes.c_char_p(self.encode(config.primaryKeyFile))
        conf.standbyKeyStoreFile = ctypes.c_char_p(self.encode(config.standbyKeyFile))
        conf.backupFolderName = ctypes.c_char_p(self.encode(config.backupDirectory))
        conf.logPath = ctypes.c_char_p(self.encode(config.logPath))
        conf.logFileName = ctypes.c_char_p(self.encode(config.logFile))
        conf.randomDevice = ctypes.c_char_p(self.encode(config.randDevice))
        conf.logLever = config.logLever
        conf.domainCount = config.domainCount
        conf.defaultAlgorithm = config.defaultAlgorithm
        conf.enableChangeFilePermission = config.enableChangeFilePermission
        conf.semKey = config.semKey
        conf.semPerm = config.semPerm
        conf.enableHw = config.enableHw
        conf.splitUserLog = config.splitUserLog
        if config.enableHw == 1:
            ak = sig_str_t()
            ak.data = ctypes.c_char_p(self.encode(config.ak))
            ak.len = len(config.ak)

            sk = sig_str_t()
            sk.data = ctypes.c_char_p(self.encode(config.sk))
            sk.len = len(config.sk)

            url = sig_str_t()
            url.data = ctypes.c_char_p(self.encode(config.kmsUrl))
            url.len = len(config.kmsUrl)

            projectId = sig_str_t()
            projectId.data = ctypes.c_char_p(self.encode(config.projectId))
            projectId.len = len(config.projectId)

            kmcSigParams = kmc_sig_params_t()
            kmcSigParams.AK = ak
            kmcSigParams.SK = sk
            kmcSigParams.host = url
            kmcSigParams.projectID = projectId

            conf.kmcSigParams = kmcSigParams
            conf.enableSSLVerify = config.enableSSLVerify
        
        return conf

    def initializeWithConfig(self, config):
        result = None
        if config is None:
            raise Exception('initialize input sccOptions is None.')
        if self.gInitFlag:
            raise Exception('Repeat to call Initialize. ')

        try:
            self.lock.acquire()
            conf = self.initConfig(config)
            self.pdll.SCC_InitializeWithConfig.argtypes = [ctypes.POINTER(configuration)]
            result = self.pdll.SCC_InitializeWithConfig(ctypes.byref(conf))
        except Exception as e:
            raise Exception('Initialize Failed. Exception=' + str(e))
        finally:
            self.lock.release()
            if result == SEC_SUCCESS:
                self.gInitFlag = True
                pass
            elif result == None:
                pass
            elif result == ErrorCode.SEC_ERR_INIT_KMC_HARD_FAIL:
                # 硬件根初始化失败，软件根初始化成功，可正常使用软件根
                self.gInitFlag = True
                raise Exception('Initialize Failed. Error=ERR_INIT_KMC_HARD_FAIL,ErrCode=' + str(result))
            else:
                raise Exception('Initialize Failed. ErrCode=' + str(result))
        return result

    def initialize(self, confFile):
        result = None
        if confFile is None:
            raise Exception('initialize input confFile is None.')
        if self.gInitFlag:
            raise Exception('Repeat to call Initialize. ')
        confFileAsc = self.encode(confFile)
        try:
            self.lock.acquire()
            result = self.pdll.SCC_Initialize(confFileAsc)
        except Exception as e:
            raise Exception('Initialize Failed. Exception=' + str(e))
        finally:
            self.lock.release()
            if result == SEC_SUCCESS:
                self.gInitFlag = True
                pass
            elif result is None:
                pass
            elif result == ErrorCode.SEC_ERR_INIT_KMC_HARD_FAIL:
                # 硬件根初始化失败，软件根初始化成功，可正常使用软件根
                self.gInitFlag = True
                raise Exception('Initialize Failed. Error=ERR_INIT_KMC_HARD_FAIL,ErrCode=' + str(result))
            else:
                raise Exception('Initialize Failed. ErrCode=' + str(result))
        return result

    def finalize(self):
        result = None
        try:
            self.lock.acquire()
            result = self.pdll.SCC_Finalize()
            self.gInitFlag = False
        except Exception as e:
            raise Exception('Finalize  Failed. Exception=' + str(e))
        finally:
            self.lock.release()
            if result != SEC_SUCCESS:
                raise Exception('Finalize  Failed. result=' + str(result))
        return result

    def createHashCode(self, plain, algorithId=AlgorithmID.SHA256):
        result = None
        hashCode = None
        if plain is None:
            raise Exception('createHashCode plain is None.')
        else:
            plainAsc = self.encode(plain)
            plainLen = len(plainAsc)
            hashResult = ctypes.c_char_p()
            hashResultLen = ctypes.c_int()
            p_hashResult = ctypes.pointer(hashResult)
            p_hashResultLen = ctypes.pointer(hashResultLen)
            try:
                result = self.pdll.SCC_CreateHashCode(algorithId, plainAsc, plainLen, p_hashResult, p_hashResultLen)
            except Exception as e:
                raise Exception('create hash code failed. Exception=' + str(e))
            finally:
                if result == SEC_SUCCESS:
                    hashCode = self.decode(hashResult.value)
                    self.pdll.SCC_Free(hashResult)
                else:
                    raise Exception('createHashCode failed. result=' + str(result))
        return hashCode

    def checkHashCode(self, plain, hashCode, algorithId=AlgorithmID.SHA256):
        result = None
        checkResult = False
        if plain is None or hashCode is None:
            raise Exception('checkHashCode plain or hashCode is None')
        else:
            plainAsc = self.encode(plain)
            plainLen = len(plainAsc)
            hashCodeAsc = self.encode(hashCode)
            hashCodeLen = len(hashCodeAsc)
            checkRet = ctypes.c_int()
            p_checkRet = ctypes.pointer(checkRet)
            try:
                result = self.pdll.SCC_CheckHashCode(algorithId, plainAsc, plainLen, hashCodeAsc, hashCodeLen, p_checkRet)
            except Exception as e:
                raise Exception('checkHashCode failed. Exception=' + str(e))
            finally:
                if result == SEC_SUCCESS:
                    if checkRet.value == SEC_TRUE:
                        checkResult = True
                else:
                    raise Exception('checkHashCode failed.result=' + str(result))
        return checkResult

    def protectPwd(self, password, algorithId=AlgorithmID.PBKDF2_HMAC_SHA256):
        result = None
        hashCode = None
        if password is None:
            raise Exception('protectPwd password is None.')
        else:
            passwordAsc = self.encode(password)
            passwordLen = len(passwordAsc)
            hashResult = ctypes.c_char_p()
            hashResultLen = ctypes.c_int()
            p_hashResult = ctypes.pointer(hashResult)
            p_hashResultLen = ctypes.pointer(hashResultLen)
            try:
                result = self.pdll.SCC_ProtectPwd(algorithId, passwordAsc, passwordLen, p_hashResult, p_hashResultLen)
            except Exception as e:
                raise Exception('protectPwd failed. Exception=' + str(e))
            finally:
                if result == SEC_SUCCESS:
                    hashCode = self.decode(hashResult.value)
                    self.pdll.SCC_Free(hashResult)
                else:
                    raise Exception('protectPwd failed. result=' + str(result))
        return hashCode

    def deleteDomain(self, domainId):
        result = None
        try:
            result = self.pdll.SCC_DeleteDomain(domainId)
        except Exception as e:
            raise Exception("deleteDomain failed. Exception=" + str(e))
        finally:
            if result != SEC_SUCCESS:
                raise Exception("deleteDomain failed result=" + str(result))
        return result

    def exportKeyByDomain(self, domainId, isHw):
        result = None
        isHwInt = 0
        if isHw:
            isHwInt = 1
        key = None
        keyRet = ctypes.c_char_p()
        keyLen = ctypes.c_int()
        p_keyRet = ctypes.pointer(keyRet)
        p_keyLen = ctypes.pointer(keyLen)
        try:
            result = self.pdll.SCC_ExportKeyByDomain(domainId, isHwInt, p_keyRet, p_keyLen)
        except Exception as e:
            raise Exception("exportKeyByDomain failed, Exception=" + str(e))
        finally:
            if result == SEC_SUCCESS:
                key = self.decode(keyRet.value)
                self.pdll.SCC_Free(keyRet)
            else:
                raise Exception("exportKeyByDomain failed, result=" + str(result))
        return key

    def importKeyByDomain(self, domainId, isHw, isEncrypt, key):
        result = None
        isHwInt = 1 if isHw else 0
        isEncryptInt = 1 if isEncrypt else 0
        if key is None:
            raise Exception("import key is None")
        keyAsc = self.encode(key)
        keyLen = len(keyAsc)
        try:
            result = self.pdll.SCC_ImportKeyByDomain(domainId, isHwInt, isEncryptInt, keyAsc, keyLen)
        except Exception as e:
            raise Exception("importKeyByDomain failed. Exception=" + str(e))
        finally:
            if result != SEC_SUCCESS:
                raise Exception("importKeyByDomain failed. result=" + str(result))
        return result

    def checkHwKey(self, cipherText):
        result = None
        checkResult = False
        if cipherText is None:
            raise Exception('checkHwKey cipherText is None')
        else:
            cipherAsc = self.encode(cipherText)
            cipherLen = len(cipherAsc)
            checkRet = ctypes.c_int()
            p_checkRet = ctypes.pointer(checkRet)
            try:
                result = self.pdll.SCC_CheckHwKey(cipherAsc, cipherLen, p_checkRet)
            except Exception as e:
                raise Exception('checkHwKey failed. Exception=' + str(e))
            finally:
                if result == SEC_SUCCESS:
                    if checkRet.value == SEC_TRUE:
                        checkResult = True
                else:
                    raise Exception('checkHwKey failed.result=' + str(result))
        return checkResult

    def getCipherAlg(self, cipherText):
        result = None
        algId = 0
        if cipherText is None:
            raise Exception('getCipherAlg cipherTex is None')
        else:
            cipherAsc = self.encode(cipherText)
            cipherLen = len(cipherAsc)
            algRet = ctypes.c_int()
            p_algRet = ctypes.pointer(algRet)
            try:
                result = self.pdll.SCC_GetCipherAlg(cipherAsc, cipherLen, p_algRet)
            except Exception as e:
                raise Exception('getCipherAlg failed. Exception=' + str(e))
            finally:
                if result == SEC_SUCCESS:
                    algId = algRet.value
                else:
                    raise Exception('getCipherAlg failed.result=' + str(result))
        return algId

    def createDomain(self, domainId):
        result = None
        try:
            result = self.pdll.SCC_CreateDomain(domainId)
        except Exception as e:
            raise Exception("createDomain failed. Exception=" + str(e))
        finally:
            if result != SEC_SUCCESS:
                raise Exception("createDomain failed result=" + str(result))
        return result

    def getMaxMkID(self, domainId):
        result = None
        maxKeyID = None
        keyID = ctypes.c_int()
        p_KeyID = ctypes.pointer(keyID)
        self.checkIfInitialized()
        try:
            result = self.pdll.SCC_GetMaxMkID(domainId, p_KeyID)
        except Exception as e:
            raise Exception('GetMaxMkID Failed. Exception=' + str(e))
        finally:
            if result == SEC_SUCCESS:
                maxKeyID = keyID.value
            else:
                raise Exception('GetMaxMkID failed. result=' + str(result))
        return maxKeyID

    def isExistMkID(self, domainId, keyID):
        result = None
        existResult = False
        self.checkIfInitialized()
        existInd = ctypes.c_int()
        p_ExistInd = ctypes.pointer(existInd)

        try:
            result = self.pdll.SCC_IsExistMkID(domainId, keyID, p_ExistInd)
        except Exception as e:
            raise Exception('IsExistMkID Failed. Exception=' + str(e))
        finally:
            if result == SEC_SUCCESS:
                if existInd.value == SEC_TRUE:
                    existResult = True
            else:
                raise Exception('IsExistMkID failed. result=' + str(result))
        return existResult

    def getPublicKey(self):
        result = None
        keyText = None
        self.checkIfInitialized()
        key = ctypes.c_char_p()
        p_key = ctypes.pointer(key)
        try:
            result = self.pdll.SCC_GetPublicKey(p_key)
        except Exception as e:
            raise Exception('GetPublicKey Failed. Exception=' + str(e))
        finally:
            if result == SEC_SUCCESS:
                keyText = self.decode(key.value)
                self.pdll.SCC_Free(key)
            else:
                raise Exception('GetPublicKey failed. result=' + str(result))
        return keyText

    def publicKeyEncrypt(self, plain, publicKey):
        result = None
        cipherText = None
        if plain is None:
            raise Exception('encrypt input plain is None.')
        self.checkIfInitialized()
        publicKeyAsc = self.encode(publicKey)
        plainAsc = self.encode(plain)
        plainLen = len(plainAsc)
        cipher = ctypes.c_char_p()
        cipherLen = ctypes.c_int()
        p_cipher = ctypes.pointer(cipher)
        p_cipherLen = ctypes.pointer(cipherLen)
        try:
            result = self.pdll.SCC_PublicKeyEncrypt(plainAsc, plainLen, publicKeyAsc, p_cipher, p_cipherLen)
        except Exception as e:
            raise Exception('PublicKeyEncrypt Failed. Exception=' + str(e))
        finally:
            if result == SEC_SUCCESS:
                cipherText = self.decode(cipher.value)
                self.pdll.SCC_Free(cipher)
            else:
                raise Exception('PublicKeyEncrypt failed. result=' + str(result))
        return cipherText

    def privateKeyDecrypt(self, cipher):
        result = None
        plainText = None
        if cipher is None:
            raise Exception('decrypt input cipher is None.')
        self.checkIfInitialized()
        cipherAsc = self.encode(cipher)
        cipherLen = len(cipherAsc)
        plain = ctypes.c_char_p()
        plainLlen = ctypes.c_int()
        p_plain = ctypes.pointer(plain)
        p_plainLlen = ctypes.pointer(plainLlen)
        try:
            result = self.pdll.SCC_PrivateKeyDecrypt(cipherAsc, cipherLen, p_plain, p_plainLlen)
        except Exception as e:
            raise Exception('PrivateKeyDecrypt Failed. Exception=' + str(e))
        finally:
            if result == SEC_SUCCESS:
                plainText = self.decode(plain.value)
                self.pdll.SCC_Free(plain)
            else:
                raise Exception('PrivateKeyDecrypt failed. result=' + str(result))
        return plainText

    def resetRSAKey(self):
        result = None
        try:
            result = self.pdll.SCC_ResetRSAKey()
        except Exception as e:
            raise Exception('ResetRSAKey Failed. Exception=' + str(e))
        finally:
            if result != SEC_SUCCESS:
                raise Exception('ResetRSAKey failed. result=' + str(result))
    def updateAccessKey(self, ak, sk):
        result = None
        if ak is None or sk is None:
            raise Exception('updateAccessKey ak or sk is None')
        else:
            akAsc = self.encode(ak)
            akLen = len(akAsc)
            skAsc = self.encode(sk)
            skLen = len(skAsc)
            try:
                result = self.pdll.SCC_UpdateAccessKey(akAsc, akLen, skAsc, skLen)
            except Exception as e:
                raise Exception('updateAccessKey failed. Exception=' + str(e))
            finally:
                if result != SEC_SUCCESS:
                    raise Exception('updateAccessKey failed. result=' + str(result))
        return result

