import ctypes
import threading
import platform
from AlgorithmID import AlgorithmID
from ErrorCode import ErrorCode

SEC_SUCCESS = 0
SEC_TRUE = 1
SEC_FALSE = 0
DEFAULT_DOMAIN_ID = 0
# 256 bit / 8 =32 byte
DEFAULT_KEY_LEN = 32
KEYID_AUTO_GET = 0


class Crypto(object):
    def __init__(self):
        self.gInitFlag = False
        self.lock = threading.Lock()
        if platform.system() == 'Windows':
            self.pdll = ctypes.cdll.LoadLibrary('C:\\Windows\\seccomponent\\lib\\x64\\sc-secrypto.dll')
        else:
            self.pdll = ctypes.cdll.LoadLibrary('/usr/local/seccomponent/lib/libsc-secrypto.so')

    def checkIfInitialized(self):
        if not self.gInitFlag:
            raise Exception('please initialize SCC first')

    def encode(self, value):
        if isinstance(value, bytes):
            return value
        else:
            return value.encode('UTF-8')

    def decode(self, value):
        if isinstance(value, bytes):
            return value.decode('UTF-8')
        else:
            return value

    def reEncrypt(self, cipher, domainId=DEFAULT_DOMAIN_ID, algorithmId=AlgorithmID.DEFAULT):
        result = None
        newCipherText = None
        if cipher is None:
            raise Exception('reEncrypt input cipher is None.')
        self.checkIfInitialized()
        cipherAsc = self.encode(cipher)
        cipherLen = len(cipherAsc)
        newCipher = ctypes.c_char_p()
        newCipherLen = ctypes.c_int()
        p_newCipher = ctypes.pointer(newCipher)
        p_newCipherLen = ctypes.pointer(newCipherLen)
        try:
            result = self.pdll.SCC_ReEncryptByAlgorithmDomain(algorithmId, domainId, cipherAsc,
                cipherLen, p_newCipher, p_newCipherLen)
        except Exception as e:
            raise Exception('ReEncryptByDomain Failed. Exception=' + str(e))
        finally:
            if result == SEC_SUCCESS:
                newCipherText = self.decode(newCipher.value)
                self.pdll.SCC_Free(newCipher)
            else:
                raise Exception('ReEncryptByDomain failed. result=' + str(result))
        return newCipherText

    def createHmacCode(self, plain, domainId=None, algorithId=AlgorithmID.HMAC_SHA256):
        result = None
        hmacCode = None
        if domainId is None:
            domainId = DEFAULT_DOMAIN_ID
        if plain is None:
            raise Exception('createHmacCode plain is None.')
        else:
            plainAsc = self.encode(plain)
            plainLen = len(plainAsc)
            hashResult = ctypes.c_char_p()
            hashResultLen = ctypes.c_int()
            p_hashResult = ctypes.pointer(hashResult)
            p_hashResultLen = ctypes.pointer(hashResultLen)
            try:
                result = self.pdll.SCC_CreateHmacCode(algorithId, domainId, plainAsc, plainLen, p_hashResult, p_hashResultLen)
            except Exception as e:
                raise Exception('createHmacCode failed. Exception=' + str(e))
            finally:
                if result == SEC_SUCCESS:
                    hmacCode = self.decode(hashResult.value)
                    self.pdll.SCC_Free(hashResult)
                else:
                    raise Exception('createHmacCode failed. result=' + str(result))
        return hmacCode

    def createFileHmacCode(self, filePath, domainId=None, algorithId=AlgorithmID.HMAC_SHA256):
        result = None
        hmacCode = None
        if domainId is None:
            domainId = DEFAULT_DOMAIN_ID
        if filePath is None:
            raise Exception('createFileHmacCode filePath is None.')
        else:
            filePathAsc = self.encode(filePath)
            hashResult = ctypes.c_char_p()
            hashResultLen = ctypes.c_int()
            p_hashResult = ctypes.pointer(hashResult)
            p_hashResultLen = ctypes.pointer(hashResultLen)
            try:
                result = self.pdll.SCC_CreateFileHmacCode(algorithId, domainId, filePathAsc, p_hashResult, p_hashResultLen)
            except Exception as e:
                raise Exception('createFileHmacCode failed. Exception=' + str(e))
            finally:
                if result == SEC_SUCCESS:
                    hmacCode = self.decode(hashResult.value)
                    self.pdll.SCC_Free(hashResult)
                else:
                    raise Exception('createFileHmacCode failed. result=' + str(result))
        return hmacCode

    def encrypt(self, plain, domainId=DEFAULT_DOMAIN_ID, algorithmId=AlgorithmID.DEFAULT):
        result = None
        cipherText = None
        if plain is None:
            raise Exception('encrypt input plain is None.')
        self.checkIfInitialized()
        plainAsc = self.encode(plain)
        plainLen = len(plainAsc)
        cipher = ctypes.c_char_p()
        cipherLen = ctypes.c_int()
        p_cipher = ctypes.pointer(cipher)
        p_cipherLen = ctypes.pointer(cipherLen)
        try:
            result = self.pdll.SCC_EncryptByAlgorithmDomain(algorithmId, domainId, plainAsc, plainLen, p_cipher,
                                                            p_cipherLen)
        except Exception as e:
            raise Exception('Encrypt Failed. Exception=' + str(e))
        finally:
            if result == SEC_SUCCESS:
                cipherText = self.decode(cipher.value)
                self.pdll.SCC_Free(cipher)
            else:
                raise Exception('Encrypt failed. result=' + str(result))
        return cipherText

    def decrypt(self, cipher, domainId=DEFAULT_DOMAIN_ID):
        result = None
        plainText = None
        if cipher is None:
            raise Exception('decrypt input cipher is None.')
        self.checkIfInitialized()
        cipherAsc = self.encode(cipher)
        cipherLen = len(cipherAsc)
        plain = ctypes.c_char_p()
        plainLlen = ctypes.c_int()
        p_plain = ctypes.pointer(plain)
        p_plainLlen = ctypes.pointer(plainLlen)
        try:
            result = self.pdll.SCC_DecryptByDomain(domainId, cipherAsc, cipherLen, p_plain, p_plainLlen)
        except Exception as e:
            raise Exception('Decrypt Failed. Exception=' + str(e))
        finally:
            if result == SEC_SUCCESS:
                plainText = self.decode(plain.value)
                self.pdll.SCC_Free(plain)
            else:
                raise Exception('Decrypt failed. result=' + str(result))
        return plainText

    def setDomain(self, domainId=DEFAULT_DOMAIN_ID):
        try:
            self.pdll.SCC_SetDomain(domainId)
            global DEFAULT_DOMAIN_ID
            DEFAULT_DOMAIN_ID = domainId
        except Exception as e:
            raise Exception('SetDomain Failed. Exception=' + str(e))

    def resetDomain(self):
        try:
            self.pdll.SCC_ResetDomain()
            global DEFAULT_DOMAIN_ID
            DEFAULT_DOMAIN_ID = 0
        except Exception as e:
            raise Exception('ResetDomain Failed. Exception=' + str(e))

    def activeNewKey(self, domainId=DEFAULT_DOMAIN_ID):
        result = None
        activeResult = False
        self.checkIfInitialized()
        try:
            result = self.pdll.SCC_ActiveNewKey(domainId)
        except Exception as e:
            raise Exception('ActiveNewKey Failed. Exception=' + str(e))
        finally:
            if result == SEC_SUCCESS:
                activeResult = True
            else:
                raise Exception('ActiveNewKey failed. result=' + str(result))
        return activeResult

    def genSecKey(self, keyLen=DEFAULT_KEY_LEN):
        result = None
        keyText = None

        key = ctypes.c_char_p()
        newkeyLen = ctypes.c_int()
        p_key = ctypes.pointer(key)
        p_keyLen = ctypes.pointer(newkeyLen)
        try:
            result = self.pdll.SCC_GenSecKey(keyLen, p_key, p_keyLen)
        except Exception as e:
            raise Exception('GenSecKey Failed. Exception=' + str(e))
        finally:
            if result == SEC_SUCCESS:
                keyText = self.decode(key.value)
                self.pdll.SCC_Free(key)
            else:
                raise Exception('GenSecKey failed. result=' + str(result))
        return keyText

    def registerKey(self, key, domainId, keyID=KEYID_AUTO_GET):
        result = None
        regResult = False
        self.checkIfInitialized()
        try:
            keyAsc = self.encode(key)
            keyLen = len(keyAsc)
            result = self.pdll.SCC_RegisterKey(domainId, keyID, keyAsc, keyLen)
        except Exception as e:
            raise Exception('RegisterKey Failed. Exception=' + str(e))
        finally:
            if result == SEC_SUCCESS:
                regResult = True
            else:
                raise Exception('RegisterKey failed. result=' + str(result))
        return regResult

    def getKey(self, domainId=DEFAULT_DOMAIN_ID, keyID=KEYID_AUTO_GET, encodeInd=True):
        result = None
        keyText = None
        self.checkIfInitialized()
        key = ctypes.c_char_p()
        keyLen = ctypes.c_int()
        p_key = ctypes.pointer(key)
        p_keyLen = ctypes.pointer(keyLen)
        encodeIndParm = SEC_TRUE
        if not encodeInd:
            encodeIndParm = SEC_FALSE
        try:
            result = self.pdll.SCC_GetKeyByID(domainId, keyID, p_key, p_keyLen, encodeIndParm)
        except Exception as e:
            raise Exception('GetKeyByID Failed. Exception=' + str(e))
        finally:
            if result == SEC_SUCCESS:
                keyText = self.decode(key.value)
                self.pdll.SCC_Free(key)
            else:
                raise Exception('GetKeyByID failed. result=' + str(result))
        return keyText

    def checkHmacCode(self, plain, hmacCode, domainId=None):
        result = None
        checkResult = False
        if domainId is None:
            domainId = DEFAULT_DOMAIN_ID
        if plain is None or hmacCode is None:
            raise Exception('checkHmacCode plain or hmacCode is None')
        else:
            plainAsc = self.encode(plain)
            plainLen = len(plainAsc)
            hashCodeAsc = self.encode(hmacCode)
            hashCodeLen = len(hashCodeAsc)
            checkRet = ctypes.c_int()
            p_checkRet = ctypes.pointer(checkRet)
            try:
                result = self.pdll.SCC_CheckHmacCode(domainId, plainAsc, plainLen, hashCodeAsc, hashCodeLen, p_checkRet)
            except Exception as e:
                raise Exception('checkHmacCode failed. Exception=' + str(e))
            finally:
                if result == SEC_SUCCESS:
                    if checkRet.value == SEC_TRUE:
                        checkResult = True
                else:
                    raise Exception('checkHmacCode failed.result=' + str(result))
        return checkResult

    def checkFileHmacCode(self, filePath, hmacCode, domainId=None):
        result = None
        checkResult = False
        if domainId is None:
            domainId = DEFAULT_DOMAIN_ID
        if filePath is None or hmacCode is None:
            raise Exception('checkFileHmacCode filePath or hmacCode is None')
        else:
            filePathAsc = self.encode(filePath)
            hashCodeAsc = self.encode(hmacCode)
            hashCodeLen = len(hashCodeAsc)
            checkRet = ctypes.c_int()
            p_checkRet = ctypes.pointer(checkRet)
            try:
                result = self.pdll.SCC_CheckFileHmacCode(domainId, filePathAsc, hashCodeAsc, hashCodeLen, p_checkRet)
            except Exception as e:
                raise Exception('checkFileHmacCode failed. Exception=' + str(e))
            finally:
                if result == SEC_SUCCESS:
                    if checkRet.value == SEC_TRUE:
                        checkResult = True
                else:
                    raise Exception('checkFileHmacCode failed.result=' + str(result))
        return checkResult

    def verifyPwd(self, password, hashCode):
        result = None
        checkResult = False
        if password is None or hashCode is None:
            raise Exception('verifyPwd password or hashCode is None')
        else:
            passwordAsc = self.encode(password)
            passwordLen = len(passwordAsc)
            hashCodeAsc = self.encode(hashCode)
            hashCodeLen = len(hashCodeAsc)
            checkRet = ctypes.c_int()
            p_checkRet = ctypes.pointer(checkRet)
            try:
                result = self.pdll.SCC_VerifyPwd(passwordAsc, passwordLen, hashCodeAsc, hashCodeLen, p_checkRet)
            except Exception as e:
                raise Exception('verifyPwd failed. Exception=' + str(e))
            finally:
                if result == SEC_SUCCESS:
                    if checkRet.value == SEC_TRUE:
                        checkResult = True
                else:
                    raise Exception('verifyPwd failed.result=' + str(result))
        return checkResult

    def updateRootKey(self):
        result = None
        try:
            result = self.pdll.SCC_UpdateRootKey()
        except Exception as e:
            raise Exception('UpdateRootKey Failed. Exception=' + str(e))
        finally:
            if result == SEC_SUCCESS:
                pass
            elif result == ErrorCode.SEC_ERR_COMMON_KMC_HARD_FAIL:
                raise Exception('Update hardware root key failed. result=' + str(result))
            else:
                raise Exception('Update root key failed. result=' + str(result))
        return result
